/*
Copyright Bourdon Jeremie 
contributors : Giraudet Etienne, Docet Eric

jeremie.bourdon@univ-nantes.fr
eric.docet@etu.univ-nantes.fr
etienne.giraudet@etu.univ-nants.fr

This software is a computer program whose purpose is to do computations
on genetic graphs.

This software is governed by the CeCILL license under French law and
abiding by the rules of distribution of free software.  You can use, 
modify and/ or redistribute the software under the terms of the CeCILL
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL license and that you accept its terms.
*/

package calcul;


import ihm.Application;
import ihm.FenetrePrincipale;
import ihm.dessin.DessinGraphe;
import ihm.intervalle.Intervalle;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.Random;
import java.util.concurrent.Semaphore;

import javax.swing.JOptionPane;



import dialog.DialogSelectionneVariable;
import dialog.ErreurDialog;

/**
 * Classe qui permet de calculer une approximation du volume de l'ensemble de solution
 * @author giraudet-docet
 *
 */
public class CalculVolume extends Thread
{
	private static final int RANDMAX = (int)Math.pow(2, 20);
	private int NB_POINTS;
	private static final double PRECISION = 0.00000001;
	private int N;
	private Random random;
	private Matrice<Double> solution;
	private Matrice<Intervalle> matriceIntervalles;
	private ArrayList<MethodeResolution> methodes;
	public Semaphore semaphore;
	public ArrayList<ArrayList<Double>> ensemblePoints;
	private Matrice<ArrayList<Double>> listefacettes;
	private int nbfaces;
	private double volume;
	private int dim;
	private static int num=0;
	private static Semaphore fichier = new Semaphore(1);
	private int numero;
	private RechercheLocale rl;

	public CalculVolume(Matrice<Intervalle> mi, int nbEtat, ArrayList<MethodeResolution> mr,int nbpoints,int _dim)
	{
		N = nbEtat;
		matriceIntervalles = mi;
		methodes = mr;
		random = new Random();
		num++;
		numero=num;
		rl = new RechercheLocale(mi, mr, DessinGraphe.EPSILON,num);
		dim = _dim;
		NB_POINTS=nbpoints;
		ensemblePoints = new ArrayList<ArrayList<Double>>();
		semaphore = new Semaphore(1);
		try {
			semaphore.acquire();
			} 
		catch (InterruptedException e) {
				e.printStackTrace();
			}
	}

	public void run()
	{
		//on cherche le points solution de depart
		rl.start();
		while(!rl.s.tryAcquire())
		{}
		solution = rl.resultat;
		//si c'est nul on arrete sinon on continue
		if(solution!=null)
		{
			long t = System.currentTimeMillis();
			if(!isInterrupted())
			{
				nbfaces=0;
				//calcul des points sur le bord de la surface
				calculNuagePoints();
				//on regarde si on peut ecrire dans le fichier
				while(!fichier.tryAcquire())
				{}


				File f = new File("test.txt");
				try 
				{
					//on ecrit les points dans les fichiers en respectant la syntaxe pour la commande qdelaunay
					FileWriter fw = new FileWriter(f);
					fw.write(""+ensemblePoints.get(0).size()+"\n");
					fw.write(ensemblePoints.size()+"\n");
					for(int i=0;i<ensemblePoints.size();i++)
					{
						String s = "";
						ArrayList<Double> l = ensemblePoints.get(i);
						for(int j=0;j<l.size();j++)
							if(!(j==(ensemblePoints.size()-1)))
								s=s+l.get(j)+" ";
							else
								s=s+l.get(j);
						s+="\n";
						fw.write(s);
					}
					fw.flush();
					fw.close();
				} 
				catch (IOException e1) 
				{
					e1.printStackTrace();
				}
				Runtime mRuntime = Runtime.getRuntime();
				mRuntime.traceInstructions(true);
				mRuntime.traceMethodCalls(true);
				char[] buffer = new char[100];
				Process mProc = null;

				try {
					mProc = mRuntime.exec("qdelaunay Qt i TI test.txt TO result");
					InputStream is = mProc.getErrorStream();
					InputStreamReader isr = new InputStreamReader(is);
					while (isr.read(buffer,0,99)!=-1)
						System.out.println(buffer);
					System.out.println(buffer);
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
		if(!isInterrupted())
		{
			//on recupere le fichier de resultat result et on construit la liste de facette
			try {
				FileReader fr = new FileReader("result");
				BufferedReader br = new BufferedReader(fr);
				String ligne;
				try {
					if((ligne=br.readLine())!=null)
					{
						nbfaces = new Integer(ligne);
						listefacettes = new Matrice<ArrayList<Double>>(nbfaces,dim+1);
					}
					for(int i=0;i<nbfaces;i++)
					{
						ligne=br.readLine();
						char[] c = ligne.toCharArray();
						int cpt=0;
						for(int j=0;j<ensemblePoints.get(0).size()+1;j++)
						{
							String s="";
							while((c[cpt]!=' ') && (c[cpt]!='\n'))
							{
								s+=c[cpt];
								cpt++;
							}
							cpt++;
							listefacettes.setCase(i, j, ensemblePoints.get(new Integer(s)));
						}
					}
				} catch (IOException e) {
					e.printStackTrace();
				}


			} catch (FileNotFoundException e) {
				e.printStackTrace();
			}
		}
		fichier.release();
		if(!isInterrupted())
		{
			//on calcule le volume des facettes si la triangularisation a pu s'effectue
			if(listefacettes!=null)
			{
				CalculVolumeFace();
			}
			else
				JOptionPane.showMessageDialog(FenetrePrincipale.getInstance(), Application.langue.getProperty("erreurv"));
			System.out.println("tps for volume n°"+num+"= "+(System.currentTimeMillis()-t));
			
		}
		}else
		{
			new ErreurDialog(Application.langue.getProperty("passolution"));
		}
	
		semaphore.release();
	}
	
	
	public void affiche3D()
	{
		double max=Double.NEGATIVE_INFINITY;
		double min=Double.POSITIVE_INFINITY;
		double max3d=1;
		double min3d=1;
		ArrayList<Double> ecart = new ArrayList<Double>();
		for(int i=0;i<ensemblePoints.get(0).size();i++)
		{
			for(int j=0;j<ensemblePoints.size();j++)
			{
				double t = ensemblePoints.get(j).get(i);
				if(t>max)
					max=t;
				if(t<min)
					min=t;
				if((1-t)<max3d)
					max3d=t;
				if(t<min3d)
					min3d=t;
			}
			ecart.add(max-min);
			max=Double.NEGATIVE_INFINITY;
			min=Double.POSITIVE_INFINITY;
		}
		int max1=0;
		int max2=1;
		int max3=2;
		for(int i=3;i<ecart.size();i++)
		{
			double t = ecart.get(i);
			if(t>ecart.get(max1))
			{
				max3=max2;
				max2=max1;
				max1=i;
			}
			else
			{
				if(t>ecart.get(max2))
				{
					max3=max2;
					max2=i;
				}
				else
					if(t>ecart.get(max3))
						max3=i;
				
			}
		}
			new DialogSelectionneVariable(N,max1,max2,max3,listefacettes,ensemblePoints);
	}
	
	public double getVolume()
	{
		return volume;
	}

	public int getnum()
	{
		return numero;
	}
	/**
	 * fonction qui genere un ensemble de points sur la frontiere
	 *
	 */
	public void calculNuagePoints()
	{
		while(ensemblePoints.size()<NB_POINTS)
		{
			ArrayList<Double> m =rechercheBordDichotomique();
			if(!ensemblePoints.contains(m))
				ensemblePoints.add(m);
		}
	}

	/**
	 * Fonction qui calcule le volume de l'ensemble solution a partir d'un ensemble de simplex 
	 *
	 */
	public void CalculVolumeFace()
	{
		volume=0;
		if(!isInterrupted())
		{
			for(int i=0;i<listefacettes.getNbLigne();i++)
			{
				if(!isInterrupted())
				{
					//on construit la matrice des distances
					Matrice<Double> m = new Matrice<Double> (dim+2,dim+2);
					for(int j=0;j<m.getNbLigne();j++)
					{
						if(!isInterrupted())
						{
							m.setCase(j, j, 0.0);
							if(j!=0)
							{
								m.setCase(0,j,1.0);
								m.setCase(j,0,1.0);
							}
						}
						else
							break;
					}
					if(!isInterrupted())
					{
						for(int j=0;j<listefacettes.getNbColonne()-1;j++)
						{
							for(int l=j+1;l<listefacettes.getNbColonne();l++)
							{
								if(!isInterrupted())
								{
									double dist=Dist(listefacettes.getCase(i, j),listefacettes.getCase(i, l));
									m.setCase(j+1,l+1,dist);
									m.setCase(l+1,j+1,dist);
								}
								else
									break;
							}
						}
					}
					else
						break;
					//on applique la formule du volume pour un simplex
					volume+=((Math.pow(-1, dim+1)/(Math.pow(2, dim)*Math.pow(factoriel(dim), 2))))*Matrice.Determinant(m);
				}
				else
					break;
			}
			volume=Math.sqrt(volume);
		}
	}

	/**
	 * fonction qui retourne une matrice situe sur la frontiere de l'ensemble solution 
	 * cette matrice est calcule a partir d'une recherche dichotomique 
	 * en construisant un segment entre un point sur le cube et le point solution 
	 * @return
	 */
	public ArrayList<Double> rechercheBordDichotomique()
	{
		Matrice<Double> m2 = genereMatriceSurCube();
		if(estSolution(m2))
			return MetMatriceBonneDimension(m2);
		else
		{
			Matrice<Double> res;
			double min = 0d;
			double max = 1d; 
			double lambda = (min+max)/2;
			int cpt=0;
			do
			{
				res = genereMatriceAvecLambda(solution, m2, lambda);
				if(estSolution(res))
					max = lambda;
				else
					min = lambda;
				lambda = (min+max)/2;
				cpt++;
			}
			while(Math.min(Math.abs(lambda-min),Math.abs(lambda-max))>PRECISION);
			return MetMatriceBonneDimension(res);
		}
	}


	/**
	 * Calcule la matrice a partir de deux matrices extrémitées du segment
	 * d'apres la formule :
	 * 		L * M1   +  (1-L) * M2 
	 * @param lambda
	 * @return
	 */
	public Matrice<Double> genereMatriceAvecLambda(Matrice<Double> m1, Matrice<Double> m2, double lambda)
	{
		Matrice<Double> res1 = multParCoefIntervalle(m1, lambda,matriceIntervalles);
		Matrice<Double> res2 = multParCoefIntervalle(m2, 1-lambda, matriceIntervalles);
		return sommeMatriceIntervalle(res1, res2,matriceIntervalles);
	}

	/**
	 * Verifie si une matrice m est solution en tenant compte de la liste des methodes de resolution
	 * @param m
	 * @return
	 */

	public boolean estSolution(Matrice<Double> m)
	{
		boolean sol = true;
		for(int i=0; i<methodes.size(); i++)
		{
			MethodeResolution mr = methodes.get(i);
			mr.prepareRechercheLocale();
			sol &= mr.estSolution(m,null);
		}
		return sol; 
	}


	/**
	 * Generation aléatoire de nMat matrices sur l'hypercube unitaire.
	 * @param nMat
	 * @return
	 */

	public ArrayList<Matrice<Double>> genereEnsembleMatriceSurCube(int nMat)
	{
		ArrayList<Matrice<Double>> res = new ArrayList<Matrice<Double>>();

		for(int i=0; i<nMat; i++)
		{
			res.add(genereMatriceSurCube());
		}		
		return res;
	}

	/**
	 * Genere une matrice sur le cube
	 * @return
	 */
	private Matrice<Double> genereMatriceSurCube()
	{
		Matrice<Double> res = new Matrice<Double>(N,N);



		int val = random.nextInt(2);
		int l = random.nextInt(N);
		int c = random.nextInt(N);

		if(matriceIntervalles.getCase(l, c).IntervalleUneValeur())
			while(matriceIntervalles.getCase(l, c).IntervalleUneValeur())
			{
				val = random.nextInt(2);
				l = random.nextInt(N);
				c = random.nextInt(N);
			}
		for(int i=0; i<N; i++)
		{
			int s = sommeIntervalleUneValeur(i);
			if(i!=l)
			{
				ArrayList<Integer> Xi = genereXi(i);
				double sommeXi = sumListe(Xi);
				int cpt = 0;
				for(int j=0; j<N; j++)
				{
					if(matriceIntervalles.getCase(i, j).IntervalleUneValeur())
						res.setCase(i, j, ((double)Xi.get(cpt)/100));
					else
						res.setCase(i, j, Xi.get(cpt)/sommeXi);
					cpt++;
				}
			}
			else
			{
				if(val==1)
				{
					for(int j=0; j<N; j++)
					{	if(j==c)
					{
						res.setCase(i, j, 1d-((double)s)/100);
					}
					else
					{	
						if(matriceIntervalles.getCase(i, j).IntervalleUneValeur())
							res.setCase(i, j, matriceIntervalles.getCase(i, j).getMinProba());
						else
							res.setCase(i, j, 0d);
					}
					}
				}
				else
				{
					ArrayList<Integer> Xi = genereXi0(N-1,c);
					double sommeXi = sumListe(Xi);
					int cpt = 0;
					for(int j=0; j<N; j++)
					{	
						if(j==c)
						{
							res.setCase(i, j, 0d);
						}
						else
						{
							if(matriceIntervalles.getCase(i, j).IntervalleUneValeur())
								res.setCase(i, j, ((double)(Xi.get(cpt)))/100);
							else
								res.setCase(i, j, Xi.get(cpt)/sommeXi-((double)s)/100);
						}
						cpt++;
					}
				}
			}
		}

		return res;
	}

	/**
	 * genere une ligne dont la somme est egale à 1
	 * @param n
	 * @return
	 */
	private ArrayList<Integer> genereXi(int n)
	{
		ArrayList<Integer> li = new ArrayList<Integer>();

		for(int i=0; i<matriceIntervalles.getNbColonne(); i++)
			if(matriceIntervalles.getCase(n,i).IntervalleUneValeur())
				li.add(matriceIntervalles.getCase(n,i).getMin());
			else
				li.add(random.nextInt(RANDMAX));
		return li;
	}

	/**
	 * genere une ligne dont la somme est egale à 1 et dont une valeur vaut 0
	 * @param n
	 * @return
	 */
	private ArrayList<Integer> genereXi0(int n,int c)
	{
		ArrayList<Integer> li = new ArrayList<Integer>();

		for(int i=0; i<matriceIntervalles.getNbColonne(); i++)
			if(i==c)
				li.add(0);
			else
				if(matriceIntervalles.getCase(n,i).IntervalleUneValeur())
					li.add(matriceIntervalles.getCase(n,i).getMin());
				else
					li.add(random.nextInt(RANDMAX));
		return li;
	}

	/**
	 * compte la somme des elements d'une liste d'entier
	 * @param li
	 * @return
	 */
	private static int sumListe(ArrayList<Integer> li)
	{
		int res=0;
		for(int i=0; i<li.size(); i++)
			res += li.get(i);
		return res;
	}

	/**
	 * compte la somme des intervalles a une valeur
	 */
	private int sommeIntervalleUneValeur(int l)
	{
		int x = 0;
		for(int i=0;i<matriceIntervalles.getNbColonne();i++)
		{
			if(matriceIntervalles.getCase(l,i).IntervalleUneValeur())
				x += matriceIntervalles.getCase(l, i).getMax();
		}
		return x;
	}

	/**
	 * Compte le nombre d'intervalle nul sur une ligne
	 * @param l
	 * @return
	 */
	/*private int nombreIntervallesNulSurLigne(int l)
	{
		int res = 0;
		for(int i=0; i<N; i++)
			if(matriceIntervalles.getCase(l, i).IntervalleNul())
				res++;
		return res;
	}*/

	/**
	 * fonction qui calcul la factorielle d'un nombre
	 * @param n
	 * @return
	 */
	public static int factoriel(int n)
	{
		int res = n;
		while(n>1)
		{
			n--;
			res *= n;  
		}
		return res;
	}


	
/*	public ArrayList<Double> MetMatriceBonneDimension1(Matrice<Double> m)
	{
		ArrayList<Double> l = new ArrayList<Double>();
		for(int i=0;i<m.getNbLigne();i++)
		{
			if(!lignebloque(i))
			{
				for(int j=0;j<m.getNbColonne();j++)
					l.add(m.getCase(i, j));
			}
		}
		return l;		
	}*/
	
	public boolean lignebloque(int i)
	{
		for(int j=0;j<matriceIntervalles.getNbColonne();j++)
			if(!matriceIntervalles.getCase(i, j).IntervalleUneValeur())
				return false;
		return true;
	}

	/**
	 * fonction qui retourne la matrice m sous une matrice de bonne dimension
	 * par exemple dans le cas d'une matrice a 9 variables ayant un domaine de taille superieure a 1
	 * la bonne dimension est 6 car les dernieres variables de chaque lignes sont dependantes des valeurs
	 * des deux autres de la ligne
	 * @param m
	 * @return
	 */
	public ArrayList<Double> MetMatriceBonneDimension(Matrice<Double> m)
	{
		ArrayList<Double> res = new ArrayList<Double>();
		for(int i=0;i<m.getNbLigne();i++)
		{
			//boolean diuv=DernierIntervalleUneValeur(i);
			for(int j=0;j<m.getNbColonne()-1;j++)
			/*{	
				if(!diuv)
				{	if(!matriceIntervalles.getCase(i, j).IntervalleUneValeur())
						res.add(m.getCase(i, j));
				}
				else
					if((j!=m.getNbColonne()-2) && !matriceIntervalles.getCase(i, j).IntervalleUneValeur())
						res.add(m.getCase(i, j));
						
			}*/
			res.add(m.getCase(i,j));
		}
		return res;
	}


	
	public boolean DernierIntervalleUneValeur(int i)
	{
		return matriceIntervalles.getCase(i, matriceIntervalles.getNbColonne()-1).IntervalleUneValeur();
	}

	/**
	 * fonction qui calcule la distance euclidienne entre deux listes de double
	 * @param m1
	 * @param t1
	 * @return
	 */
	public double Dist(ArrayList<Double> liste1,ArrayList<Double> liste2)
	{
		double dis=0d;
		for(int i=0;i<liste1.size();i++)
			dis+=Math.pow(liste1.get(i)-liste2.get(i),2);
		return dis;
	}


	/**
	 * fonction qui multiplie par un coefficient les cases dont l'intervalle a plus d'une valeur
	 * @param m1
	 * @param coef
	 * @param mi
	 * @return
	 */
	public static Matrice<Double> multParCoefIntervalle(Matrice<Double> m1,double coef,Matrice<Intervalle> mi)
	{
		Matrice<Double> res= new Matrice<Double>(m1.getNbLigne(),m1.getNbColonne());
		for(int i=0;i<mi.getNbLigne();i++)
			for(int j=0;j<mi.getNbColonne();j++)
				if(!mi.getCase(i, j).IntervalleUneValeur())
					res.setCase(i, j, m1.getCase(i, j)*coef);
				else
					res.setCase(i, j, m1.getCase(i,j));
		return res;
	}

	/**
	 * fonction qui effectue la somme de deux matrices dont les cases ont un intervalle de plus d'une valeur 
	 * @param m1
	 * @param m2
	 * @param mi
	 * @return
	 */
	public static Matrice<Double> sommeMatriceIntervalle(Matrice<Double> m1,Matrice<Double> m2,Matrice<Intervalle> mi)
	{
		Matrice<Double> res= new Matrice<Double>(m1.getNbLigne(),m1.getNbColonne());
		for(int i=0;i<mi.getNbLigne();i++)
			for(int j=0;j<mi.getNbColonne();j++)
				if(!mi.getCase(i, j).IntervalleUneValeur())
					res.setCase(i, j, m1.getCase(i, j)+m2.getCase(i, j));
				else
					res.setCase(i, j, m1.getCase(i,j));
		return res;
	}
}
