/*
Copyright Bourdon Jeremie 
contributors : Giraudet Etienne, Docet Eric

jeremie.bourdon@univ-nantes.fr
eric.docet@etu.univ-nantes.fr
etienne.giraudet@etu.univ-nants.fr

This software is a computer program whose purpose is to do computations
on genetic graphs.

This software is governed by the CeCILL license under French law and
abiding by the rules of distribution of free software.  You can use, 
modify and/ or redistribute the software under the terms of the CeCILL
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL license and that you accept its terms.
 */

package calcul;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.concurrent.Semaphore;

import cout.CoutMult;
/**
 * Cette classe permet de representer la courbe d'evolution des proteines produites par les
 * genes
 * @author giraudet
 *
 */
public class CourbeGnuPlot extends Thread{

	private static Semaphore spoint=new Semaphore(1);//semaphore pour indique si les fichiers ou on ecrit les points sont libres

	public Semaphore sfin; //semaphore pour indiquer que le calcul est fini
	private int _num;
	private ArrayList<CoutMult> _lcm;
	private Matrice<Double> _lm;

	public CourbeGnuPlot(ArrayList<CoutMult> lcm, Matrice<Double> lm,int num) throws IOException
	{
		_num=num;
		_lcm=lcm;
		_lm=lm;
		sfin=new Semaphore(1);
		try {
			sfin.acquire();
		} catch (InterruptedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	public void run()
	{
		ArrayList<Matrice<Double>> _lmc = new ArrayList<Matrice<Double>>();
		while(!spoint.tryAcquire()){if(isInterrupted()) break;}
		//on multiplie les coefficient de la matrice resultat par les coefficients contenu dans la matrice de cout correspondante
		if(!isInterrupted())
		{
			for(int i=0;i<_lcm.size();i++)
			{
				if(isInterrupted()) break;
				Matrice<Double> mc = new Matrice<Double>(_lm.getNbLigne(),_lm.getNbColonne());
				Matrice<Double> c = _lcm.get(i).getMatrice();
				for(int j=0;j<_lm.getNbLigne();j++)
					for(int l=0;l<_lm.getNbColonne();l++)
						mc.setCase(j, l, _lm.getCase(j, l)*c.getCase(j, l));
				_lmc.add(mc);
			}
			double xmin=0;
			double xmax=100;
			double ymin=Double.NEGATIVE_INFINITY;
			double ymax=Double.POSITIVE_INFINITY;
			boolean first=true;
			//on calcule les points pour chaque cout en multipliant la matrice par elle meme 100 fois
			//il y a autant de points+chiffre.txt que de cout
			if(!isInterrupted())
			{
				for(int t=0;t<_lcm.size();t++)
				{
					if(isInterrupted()) break;
					CoutMult c = _lcm.get(t);
					File f = new File("points"+t+".txt");
					FileWriter fw;
					try {
						fw = new FileWriter(f);
						if(isInterrupted()) break;
						Matrice<Double> m = _lmc.get(t);
						Matrice<Double> m1 = new Matrice<Double>(m);
						ArrayList<Double> l1 = new ArrayList<Double>();
						for(int i=0;i<m.getNbColonne();i++)
							l1.add(1d);
						for(int i=0;i<100;i++)
						{
							if(i!=0)
								m1=Matrice.multMatriceReel(m1, m);
							ArrayList<Double> l2 = Matrice.multListColonne(m1,l1);
							double res=0;
							for(int j=0;j<l2.size();j++)
								res+=l2.get(j)*c.getVecteurProba().get(j);
							if (first)
							{
								ymin=res;
								ymax=res;
								first=false;
							}
							else
							{	
								if(ymin>res)
									ymin=res;
								if(ymax<res)
									ymax=res;
							}

							fw.write(""+(float)i+" ");
							fw.write(res+"\n");
							fw.flush();
						} 

						fw.close();
					} 
					catch (IOException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
				}

				if(!isInterrupted())
				{
					//on construit le fichier que gnuplot va appeler
					File f = new File("pnggraph.txt");
					File f2 = new File("psgraph.txt");
					Gnuplot(f, false, xmin, xmax, ymin, ymax);
					Gnuplot(f2, true, xmin, xmax, ymin, ymax);
					
				}
			}
		}
	}

	public void Gnuplot(File f,boolean ps,double xmin,double xmax,double ymin,double ymax)
	{
		try {
			FileWriter fw = new FileWriter(f);
			fw.write("set title 'Graph'\n");
			fw.write("set grid\n");
			fw.write("set xrange ["+xmin+":"+xmax+"]\n");
			if((ymin!=Double.NEGATIVE_INFINITY)&&(ymax!=Double.POSITIVE_INFINITY))
			{
			fw.write("set yrange ["+ymin+":"+ymax+"]\n");
			fw.write("set xtics "+xmin+","+(xmax-xmin)/10+","+xmax+"\n");
			fw.write("set ytics "+ymin+","+(ymax-ymin)/10+","+ymax+"\n");
			}
			else
			{
				if(ymin==Double.NEGATIVE_INFINITY)
					if(ymax==Double.POSITIVE_INFINITY)
					{
						fw.write("set yrange ["+0+":"+0+"]\n");
						fw.write("set xtics "+xmin+","+(xmax-xmin)/10+","+xmax+"\n");
						fw.write("set ytics "+0+","+0+","+0+"\n");
					}
					else
					{
						fw.write("set yrange ["+0+":"+ymax+"]\n");
						fw.write("set xtics "+xmin+","+(xmax-xmin)/10+","+xmax+"\n");
						fw.write("set ytics "+0+","+ymax/10+","+ymax+"\n");
					}
				else
				{
					fw.write("set yrange ["+ymin+":"+0+"]\n");
					fw.write("set xtics "+xmin+","+(xmax-xmin)/10+","+xmax+"\n");
					fw.write("set ytics "+ymin+","+(0-ymin)/10+","+0+"\n");
				}
			}
			fw.write("set function style lines\n");
			if(ps)
			{
				fw.write("set terminal postscript\n");
				fw.write("set output 'graph"+_num+".ps'\n");
			}
			else
			{
				fw.write("set terminal png\n");
				fw.write("set output 'graph"+_num+".png'\n");
			}
			fw.write("set multiplot\n");
			String s="plot ";
			for(int i=0;i<_lcm.size();i++)
			{
				if(isInterrupted())break;
				if(i<_lcm.size()-1)
					s+="'points"+i+".txt' title '"+_lcm.get(i).getNom()+"' with lines,";
				else
					s+="'points"+i+".txt' title '"+_lcm.get(i).getNom()+"' with lines\n";
			}
			if(!isInterrupted())
			{
				fw.write(s);
				fw.write("set nomultiplot\n");
				fw.flush();
				fw.close();
				Runtime mRuntime = Runtime.getRuntime();
				mRuntime.traceInstructions(true);
				mRuntime.traceMethodCalls(true);
				try {
					mRuntime.exec("gnuplot "+f.getName());
				} catch (IOException e) {
					e.printStackTrace();
				}
				spoint.release();
				sfin.release();
			}
		} catch (IOException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}
	}
	
	
	
	public void interrupt()
	{
		spoint.release();
	}
}
