/*
Copyright Bourdon Jeremie 
contributors : Giraudet Etienne, Docet Eric

jeremie.bourdon@univ-nantes.fr
eric.docet@etu.univ-nantes.fr
etienne.giraudet@etu.univ-nants.fr

This software is a computer program whose purpose is to do computations
on genetic graphs.

This software is governed by the CeCILL license under French law and
abiding by the rules of distribution of free software.  You can use, 
modify and/ or redistribute the software under the terms of the CeCILL
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL license and that you accept its terms.
*/

package calcul;

import graphe.Graphe;
import ihm.intervalle.Intervalle;
import java.util.ArrayList;
import java.util.Random;
import java.util.concurrent.Semaphore;

/**
 * Classe qui permet de calculer le nombre de solution en suivant la methode de monte-carlo
 * @author giraudet-docet
 *
 */
public class EnsembleSolution extends Thread
{
	public static final int NOMBRE_MATRICES = 10000;
	private static final int RANDMAX = (int)Math.pow(2, 20);
	private int N;
	private Random random;
	private Graphe graphe;
	private static Matrice<Intervalle> mi;
	private ArrayList<MethodeResolution> methodes;
	private int resultat;
	private static int num;
	public Semaphore semaphore;
	
	public EnsembleSolution(Graphe g, ArrayList<MethodeResolution> meth)
	{
		super();
		num++;
		random = new Random();
		graphe = g;	
		methodes = meth;
		mi = graphe.construireMatriceIntervalle();
		N = mi.getNbColonne();
		semaphore = new Semaphore(1);
		try {
			semaphore.acquire();
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
	}
	
	public int getnum()
	{
		return num;
	}
	
	public EnsembleSolution(Graphe g, ArrayList<MethodeResolution> meth, Matrice<Intervalle> m)
	{
		super();
		num++;
		mi=m;
		random = new Random();
		graphe = g;	
		methodes = meth;
		N = mi.getNbColonne();
		semaphore = new Semaphore(1);
		try {
			semaphore.acquire();
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
	}
	
	public void run()
	{
		resultat = calculNombreSolutions();
		semaphore.release();
	}	
	
	public int calculNombreSolutions()
	{
		int valid = 0;
		for(int i=0; i<NOMBRE_MATRICES; i++)
		{
			if(isInterrupted())
				break;
			else
			{
				//on genere une matrice aleatoire et on regarde si elle est solution
			Matrice<Double> m=matriceAleatoire();
			if(estDansIntervalle(m))
			{
				if(estSolution(m))
					valid++;
			}
			else
				i--;
			}
		}
		return valid;
	}
	
	public int getResult()
	{
		return resultat;
	}
	
	/**
	 * on genere aleatoirement une matrice en suivant une loi uniforme de probabilite
	 * @return Matrice<Double>
	 */
	public Matrice<Double> matriceAleatoire()
	{
		Matrice<Double> res = new Matrice<Double>(N,N);
		for(int i=0; i<N; i++)
		{
			ArrayList<Integer> Xi = genereXi(i);
			double sommeXi = sumListe(Xi,i);
			for(int j=0; j<N; j++)
			{
				if(mi.getCase(i,j).IntervalleUneValeur())
					res.setCase(i, j, ((double)(Xi.get(j)))/100);
				else
					res.setCase(i, j, (((double)(Xi.get(j))) / sommeXi));
			}
		}
		return res;
	}
		
	public ArrayList<Integer> genereXi(int ligne)
	{
		ArrayList<Integer> li = new ArrayList<Integer>(N);
		for(int i=0; i<N; i++)
			if(mi.getCase(ligne,i).IntervalleUneValeur())
				li.add(mi.getCase(ligne,i).getMin());
			else
				li.add(random.nextInt(RANDMAX));
		return li;
	}
	
	public static int sumListe(ArrayList<Integer> li,int ligne)
	{
		int res=0;
		for(int i=0; i<li.size(); i++)
			if(mi.getCase(ligne,i).IntervalleUneValeur())
				res-=li.get(i);
			else
				res += li.get(i);
		return res;
	}
	
	public boolean estSolution(Matrice<Double> m)
	{
		boolean sol = true;
		for(int i=0; i<methodes.size(); i++)
		{
			MethodeResolution mr = methodes.get(i);
			mr.prepareRechercheLocale();
			sol &= mr.estSolution(m,null);
		}
		return sol; 
	}
	
	public boolean estDansIntervalle(Matrice<Double> m)
	{
		for(int i=0;i<mi.getNbLigne();i++)
			for(int j=0;j<mi.getNbColonne();j++)
				if(!mi.getCase(i, j).estDansIntervalle(m.getCase(i, j)*100))
					return false;
		return true;
	}
	
	
	
}
