/*
Copyright Bourdon Jeremie 
contributors : Giraudet Etienne, Docet Eric

jeremie.bourdon@univ-nantes.fr
eric.docet@etu.univ-nantes.fr
etienne.giraudet@etu.univ-nants.fr

This software is a computer program whose purpose is to do computations
on genetic graphs.

This software is governed by the CeCILL license under French law and
abiding by the rules of distribution of free software.  You can use, 
modify and/ or redistribute the software under the terms of the CeCILL
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL license and that you accept its terms.
*/

package calcul;

import java.io.Serializable;
import java.util.ArrayList;

import graphe.Graphe;
import ihm.dessin.DessinGraphe;
import ihm.transition.RelationProba;
/**
 * Classe correspondant a l'ensemble des methodes de resolution
 * @author giraudet-docet
 *
 */
public abstract class MethodeResolution implements Serializable
{
	protected double EPSILON;
	protected transient RechercheLocale rechercheLocale;
	protected ArrayList<RelationProba> relations;
	protected int N = 0;
	protected Graphe _g;
	protected static int cpt=0;

	public MethodeResolution(Graphe g)
	{
		EPSILON = DessinGraphe.EPSILON;
		_g=g;
	}
	public void setRechercheLocale(RechercheLocale rl)
	{
		rechercheLocale = rl;
	}

/**
 * Methode qui permet de savoir tel ou tel voisins calcule dans la recherche locale est meilleur
 * @param m
 * @return
 */
	public abstract double heuristique(Matrice<Double> m);
	/**
	 * Methode qui determine si une matrice est solution
	 * @param m
	 * @param vecteurKCalcule
	 * @return
	 */
	public abstract boolean estSolution(Matrice<Double> m,ArrayList<Double> vecteurKCalcule);

	/**
	 * Methode a appele avant d'effectuer une recherche locale
	 *
	 */
	public void prepareRechercheLocale()
	{
		cpt++;
		if(relations==null)
			relations = _g.listeRelations;
		if(N==0)
			N = _g.getNombreEtats();
	}
	
	/**
	 * Methode a appele a la fin d'une recherche locale
	 *
	 */
	public void finRechercheLocale()
	{
		cpt--;
		if(cpt==0)
		{
			relations=null;
			N=0;
		}
	}


	public abstract String toString();
}
