/*
Copyright Bourdon Jeremie 
contributors : Giraudet Etienne, Docet Eric

jeremie.bourdon@univ-nantes.fr
eric.docet@etu.univ-nantes.fr
etienne.giraudet@etu.univ-nants.fr

This software is a computer program whose purpose is to do computations
on genetic graphs.

This software is governed by the CeCILL license under French law and
abiding by the rules of distribution of free software.  You can use, 
modify and/ or redistribute the software under the terms of the CeCILL
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL license and that you accept its terms.
*/

package calcul;


import java.util.ArrayList;

import graphe.Graphe;
import cout.CoutP2;

/**
 * Classe correspondant au calcul de la probabilite de passer une seule fois par un etat
 * @author giraudet-docet
 *
 */
public class ProbabilitePasseUneSeuleParUnEtat extends MethodeResolution
{
	private static final long serialVersionUID = 1L;
	private CoutP2 cout;
	private double resEquation; //resultat de l'équation
	
	public ProbabilitePasseUneSeuleParUnEtat(CoutP2 c,Graphe g)
	{
		super(g);
		cout = c;
	}

	@Override
	public boolean estSolution(Matrice<Double> m,ArrayList<Double> res) 
	{
		calculEquation(m);
		if(res!=null)
		res.add(resEquation);
		return cout.getIdeal().estDansIntervalle(resEquation) && Graphe.respectRelation(relations,m);
	}

	

	@Override
	public double heuristique(Matrice<Double> m) 
	{
		double valIdeal = cout.getIdeal().moyenne();
		calculEquation(m);
		return Math.abs(valIdeal - resEquation)+Graphe.evalueMatrice(relations,m);
	}
	
	/**
	 * Utilisation de la formule phi'(0)*lambda(0)^n + n*phi(0)*lambda(0)*lambda(0)^(n-1)
	 * @param m
	 */
	public void calculEquation(Matrice<Double> m)
	{
		double lambdaDeZero = cout.genereMatriceCout(m, 0.0).CalculValeurPropre();
		double lambdaDeEps = cout.genereMatriceCout(m, EPSILON).CalculValeurPropre();
		double lambdaPrimeDeZero = (lambdaDeEps-lambdaDeZero)/EPSILON;
		
		double premierMembre = phiPrimeDe0(m)*Math.pow(lambdaDeZero, cout.getDuree());
		
		double secondMembre = cout.getDuree() * phiDeZero(m) * lambdaPrimeDeZero * Math.pow(lambdaDeZero, cout.getDuree()*1);
		
		resEquation = premierMembre + secondMembre;
	}
	
	public double phiPrimeDe0(Matrice<Double> m)
	{
		double phiEps = Matrice.calculPhi(cout.genereMatriceCout(m, EPSILON), N);
		double phiZero = phiDeZero(m);
		return (phiEps-phiZero)/EPSILON;
	}
	
	public double phiDeZero(Matrice<Double> m)
	{
		return Matrice.calculPhi(cout.genereMatriceCout(m, 0d), N);
	}

	@Override
	public String toString() 
	{
		return cout.getNom();
	}

	@Override
	public void prepareRechercheLocale() {
		super.prepareRechercheLocale();
		
	}

}
