/*
Copyright Bourdon Jeremie 
contributors : Giraudet Etienne, Docet Eric

jeremie.bourdon@univ-nantes.fr
eric.docet@etu.univ-nantes.fr
etienne.giraudet@etu.univ-nants.fr

This software is a computer program whose purpose is to do computations
on genetic graphs.

This software is governed by the CeCILL license under French law and
abiding by the rules of distribution of free software.  You can use, 
modify and/ or redistribute the software under the terms of the CeCILL
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL license and that you accept its terms.
*/

package calcul;

import ihm.Application;
import ihm.intervalle.Intervalle;
import java.util.ArrayList;
import java.util.Random;
import java.util.concurrent.Semaphore;

import dialog.ErreurDialog;
import util.ListeFIFO;
/**
 * Classe servant  a lancer une recherche locale en suivant l'alogrithme du tabu search
 * @author Docet Giraudet
 *
 */
public class RechercheLocale extends Thread
{
	private static int NB_RANDOM_START = Integer.valueOf(Application.config.getProperty("randomrestart"));;
	private static double PAS = Double.valueOf(Application.config.getProperty("pas"));;
	private static int NB_ITERATIONS =Integer.valueOf(Application.config.getProperty("nbiteration"));;
	private static int TAILLE_LISTE = Integer.valueOf(Application.config.getProperty("tailleliste"));;
	private ListeFIFO<Matrice> configVue; 
	private int num;
	private Matrice<Intervalle> matriceIntervalles;
	private double EPSILON;
	public Matrice<Double> resultat;
	private ArrayList<MethodeResolution> resolution;
	public Semaphore s;
	private ArrayList<Double> vecteurKCalcule;


	/**
	 * Constructeur de la recherche locale 
	 * @param g
	 * @param mi
	 * @param mr
	 * @param eps
	 */
	public RechercheLocale(Matrice<Intervalle> mi, ArrayList<MethodeResolution> mr, double eps,int _num)
	{
		configVue = new ListeFIFO<Matrice>(TAILLE_LISTE);
		matriceIntervalles = mi;
		EPSILON=eps;
		resolution = mr;
		for(int i=0;i<resolution.size();i++)
		{
			resolution.get(i).prepareRechercheLocale();
			resolution.get(i).setRechercheLocale(this);
		}
		s = new Semaphore(1);
		try {
			s.acquire();
		} catch (InterruptedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		num=_num;
	}

	/**
	 * Fonction qui sert à modifier les parametres de la recherche locale
	 * @param nbrs
	 * @param pas
	 * @param nbi
	 * @param tl
	 */
	public static void setParametre(int nbrs,double pas,int nbi,int tl)
	{
		NB_RANDOM_START=nbrs;
		PAS=pas;
		NB_ITERATIONS=nbi;
		TAILLE_LISTE=tl;
	}

	public static int getRS()
	{
		return NB_RANDOM_START;
	}

	public static double getPas()
	{
		return PAS;
	}

	public static int getNBI()
	{
		return NB_ITERATIONS;
	}

	public static int getTL()
	{
		return TAILLE_LISTE;
	}


	/**
	 * fonction qui l'algorithme de tabou search et qui une matrice qui
	 * est solution de la recherche demandee
	 * @return Matrice<Double>
	 */
	public void run()
	{
		long dep=System.currentTimeMillis();
		int cptRS = 0;
		if(CalculPossible())
		{
			//on choisit une matrice au hasard
			Matrice<Double> m = ChoixValeur();
			if(m==null)
			{
				new ErreurDialog("Calcul n°"+num+Application.langue.getProperty("paspointdepart"));

				resultat=null;
			}
			else
			{
				//on l'ajoute a la file contenant les n dernieres matrices parcourues
				while(!estSolution(m) && cptRS <= NB_RANDOM_START)
				{
					if(!isInterrupted())
					{
						configVue.add(new Matrice<Double>(m));
						int cptIT =0;
						while(!estSolution(m) && cptIT <= NB_ITERATIONS)
						{
							if(!isInterrupted())
							{//on recupere le meilleur voisin des voisins de m
								m = calculMeilleurVoisin(calculEnsembleVoisins(m));
								if(m==null)
									cptIT = NB_ITERATIONS + 1 ;
								else
								{
									//on ajoute la nouvelle matrice a la file
									configVue.add(new Matrice<Double>(m));
									cptIT++;
								}
							}
							else
								break;
						}
						if(!estSolution(m))
							//on recommence
						{	
							m = ChoixValeur();
							int z=0;
							while(configVue.contientMatriceDouble(m, EPSILON)&&z<50)
							{
								m=ChoixValeur();
								z++;
							}
							if(z==50)
								cptRS=NB_RANDOM_START+1;
						}
						cptIT=0;
						cptRS++;
					}
					else
						break;
				}
				if(!estSolution(m)||isInterrupted())
					resultat=null;
				else
				{
					resultat=m;
				}
			}
		}
		else
		{
			new ErreurDialog("Calcul n°"+num+Application.langue.getProperty("calculimpossible"));
			resultat=null;
		}
		
		for(int i=0;i<resolution.size();i++)
			resolution.get(i).finRechercheLocale();
		System.out.println("tps exec du calcul n°"+num+" : "+(System.currentTimeMillis()-dep) + " ms");
		s.release();
	}

	public ArrayList<Double> getVecteurKCalcule()
	{
		return vecteurKCalcule;
	}

	public double getEpsilon()
	{
		return EPSILON;
	}
	public ArrayList<MethodeResolution> getMethodeResolution()
	{
		return resolution;
	}

	/**
	 * fonction qui retourne le meilleur voisin trouve en fonction de l'heuristique
	 * @param lvoisins
	 * @return
	 */
	public Matrice<Double> calculMeilleurVoisin(ArrayList<Matrice<Double>> lvoisins)
	{
		double MeilleurDist=Double.POSITIVE_INFINITY;
		ArrayList<Integer> lpos = new ArrayList<Integer>();
		for(int i=0;i<lvoisins.size();i++)
		{
			Matrice<Double> m = lvoisins.get(i);
			double h = heuristique(m);

			if(h<MeilleurDist)
			{
				//heuristique meilleur
				MeilleurDist=h;
				lpos.clear();
				lpos.add(i);
			}
			else if(h==MeilleurDist)
			{
				lpos.add(i);
			}
		}
		if(lpos.size()==0)
			return null;
		else
		{
			//on choisit au hasard parmis les meilleurs voisins trouvés
			Random r =  new Random();
			int x = r.nextInt(lpos.size());
			return lvoisins.get(lpos.get(x));
		}
	}


	/**
	 * fonction qui retourne l'ensemble des voisins de m
	 * @param m
	 * @return
	 */
	public ArrayList<Matrice<Double>> calculEnsembleVoisins(Matrice<Double> m)
	{
		ArrayList<Matrice<Double>> result = new ArrayList<Matrice<Double>>();
		for(int i=0; i<m.getNbLigne(); i++)
		{
			for(int j=0; j<m.getNbColonne(); j++)
			{
				double valMij =  m.getCase(i, j);
				//Soustraction du pas a une case
				if(valMij-PAS>=matriceIntervalles.getCase(i,j).getMinProba())
				{
					for(int k=0; k<m.getNbLigne(); k++)
					{
						double valMik = m.getCase(i,k);
						if(k!=j && valMik+PAS<=matriceIntervalles.getCase(i,k).getMaxProba())
						{
							Matrice<Double> m2 = new Matrice<Double>(m);
							m2.setCase(i, j, valMij-PAS);
							m2.setCase(i, k, valMik + PAS);
							if(!configVue.contientMatriceDouble(m2,EPSILON) && !contientMatriceDouble(result,m2))
							{
								result.add(m2);
							}
						}
					}
				}

				//Ajout du pas a une case
				if(valMij+PAS<=matriceIntervalles.getCase(i,j).getMaxProba())
				{
					for(int k=0; k<m.getNbLigne(); k++)
					{
						double valMik = m.getCase(i,k);
						if(k!=j && valMik-PAS>=matriceIntervalles.getCase(i,k).getMinProba())
						{
							Matrice<Double> m2 = new Matrice<Double>(m);
							m2.setCase(i, j, valMij + PAS);
							m2.setCase(i, k, valMik - PAS);
							if(!configVue.contientMatriceDouble(m2,EPSILON) && !contientMatriceDouble(result,m2))
							{
								result.add(m2);
							}
						}
					}
				}


			}
		}
		return result;
	}

	//Raccourcis d'appel de fonctions
	public boolean estSolution(Matrice<Double> m)
	{

		if(m!=null)
		{
			boolean b = true;
			vecteurKCalcule = new ArrayList<Double>();
			for(int i=0;i<resolution.size();i++)
			{
				if(!resolution.get(i).estSolution(m,vecteurKCalcule))
					return false;
			}
			return b;
		}
		else
			return false;
	}
	public double heuristique(Matrice<Double> m)
	{
		double somme =0.0;
		for(int i=0;i<resolution.size();i++)
			somme += resolution.get(i).heuristique(m);
		return somme;
	}






	/**
	 * fonction qui teste si une matrice est valide
	 * une matrice est valide si la somme sur chaque ligne est egal à 100
	 * @param m
	 * @return
	 */
	public static boolean MatriceValide(Matrice<Double> m)
	{
		for(int j=0;j<m.getNbLigne();j++)
			if(Matrice.sommeligne(m,j)!=100.0)
				return false;
		return true;
	}


	/**
	 * Verifie si le calcul est possible dans la matrice
	 * Le calcul est possible si la somme des minimums des intervalles dans chaque ligne est inferieur ou égal à 100
	 * et la somme des maximums des intervalles dans chaque ligne est supérieur ou égal à 100
	 * @param m
	 * @return boolean
	 */
	public boolean CalculPossible()
	{
		Intervalle inter;
		for(int i=0;i<matriceIntervalles.getNbLigne();i++)
		{
			inter=new Intervalle(0,0);
			for(int j=0;j<matriceIntervalles.getNbColonne();j++)
			{
				if(matriceIntervalles.getCase(i,j)!=null)
					inter = inter.somme(matriceIntervalles.getCase(i,j));
			}
			if((Math.round(inter.getMinDouble())>100)||(Math.round(inter.getMaxDouble())<100))
			{
				return false;
			}
		}
		return true;
	}

	/**
	 * Verifie si la valeur donnée est correcte 
	 * @param m
	 * @param d
	 * @param ResteValeur
	 * @param ligne
	 * @param ValeurSuivanteNulle
	 * @return
	 */
	@SuppressWarnings("static-access")
	public static boolean ValeurValide(Matrice<Double> m , Double d,boolean ResteValeur,int ligne,boolean ValeurSuivanteNulle )
	{

		if(ResteValeur&&!ValeurSuivanteNulle)
		{
			double somme = m.sommeligne(m,ligne);
			return somme+d<1;
		}
		else
		{
			double somme = m.sommeligne(m,ligne);
			return (new Double(somme+d).floatValue()==1)&&(new Double(somme+d).floatValue()==1);
		}
	}

	/**
	 * fonction qui détermine si l'intervalle contenu dans la matrice à la ligne i
	 * colonne j est le dernier intervalle non nul
	 * @param m
	 * @param i
	 * @param j
	 * @return boolean
	 */
	private static boolean DernierIntervalleNonNul(Matrice<Intervalle> m,int i,int j)
	{
		for(int l=j+1;l<m.getNbColonne();l++)
		{
			if(!m.getCase(i, l).IntervalleNul())
				return false;
		}
		return true;
	}

	/**
	 * fonction qui retourne la somme de ligne i jusqu'a la colonne j d'une matrice
	 * @param m
	 * @param i
	 * @param j
	 * @return float
	 */
	private static float SommeLigneALaCaseJ(Matrice<Double> m,int i,int j)
	{
		float somme=0;
		for(int l=0;l<j;l++)
			somme=somme+m.getCase(i,l).floatValue();
		return somme;
	}

	/**
	 * fonction qui regarde si l'intervalle contenu dans la case i,j de la matrice m est
	 * le dernier intervalle contenant plusieurs valeurs
	 * @param m
	 * @param i
	 * @param j
	 * @return
	 */
	private static boolean DernierIntervalleAyantPlusieursValeurs(Matrice<Intervalle> m,int i,int j)
	{
		for(int l=j+1;l<m.getNbColonne();l++)
		{
			Intervalle inter = m.getCase(i, l);
			if((!inter.IntervalleNul())&&(!inter.IntervalleUneValeur()))
				return false;
		}
		return true;
	}

	/**
	 * fonction qui test s'il y a un intervalle contenant une valeur egale a un
	 * @param m
	 * @param i
	 * @return
	 */
	private static boolean UnIntervalleAyantUneValeurEgaleAUn(Matrice<Intervalle> m,int i)
	{
		for(int l=0;l<m.getNbColonne();l++)
		{
			Intervalle inter = m.getCase(i,l);
			if(inter.IntervalleUneValeur()&&inter.getMinDouble()==100)
				return true;
		}
		return false;
	}

	/**
	 * fonction qui calcule la somme des intervalles contenant une seule valeur
	 * @param m
	 * @param i
	 * @return
	 */
	private static double SommeDesIntervallesAUneValeur(Matrice<Intervalle>m,int i)
	{
		double res=0.0;
		for(int l=0;l<m.getNbColonne();l++)
		{
			Intervalle inter = m.getCase(i,l);
			if(inter.IntervalleUneValeur())
				res+=inter.getMinProba();
		}
		return res;
	}

	/**
	 * fonction qui la somme dans la ligne i dans la matrice m jusqu'a la colonne j
	 * puis qui ajoute à la somme la valeur contenue dans les intervalles contenant une seule valeur 
	 * @param m
	 * @param i
	 * @param j
	 * @param mi
	 * @return
	 */
	private static float SommeLigneSurLaligneSauflaCaseJ(Matrice<Double> m,int i,int j,Matrice<Intervalle> mi)
	{
		float somme=0;
		for(int l=0;l<j;l++)
			somme=somme+m.getCase(i,l).floatValue();
		for(int l=j+1;l<mi.getNbColonne();l++)
			somme=somme+mi.getCase(i, l).getMinProba().floatValue();
		return somme;
	}

	/**
	 * Permet de créer une matrice en choisissant un nombre au hasard dans chaque intervalle et
	 * respectant les contraintes
	 * pre : Calcul Possible
	 * @param m
	 * @return
	 */
	public Matrice<Double> ChoixValeur()
	{
		Matrice<Double> m1 = new Matrice<Double>(matriceIntervalles.getNbColonne(),matriceIntervalles.getNbLigne());
		int iter=0;
		for(int i=0;i<matriceIntervalles.getNbLigne();i++)
		{
			boolean unintervalle=UnIntervalleDansLigne(matriceIntervalles, i);
			boolean unintervalleaunevaleuregalea1=UnIntervalleAyantUneValeurEgaleAUn(matriceIntervalles, i);
			double sommeIntervalleFixe= SommeDesIntervallesAUneValeur(matriceIntervalles, i);
			boolean som = (sommeIntervalleFixe>0);
			for(int j=0;j<matriceIntervalles.getNbColonne();j++)
			{
				Intervalle inter = matriceIntervalles.getCase(i,j);
				int z=0;
				if(inter!=null)
				{
					if(unintervalle && !inter.IntervalleNul())
					{
						//la valeur est forcement 1
						m1.setCase(i,j,(double)1);
					}
					else
					{
						if(inter.IntervalleUneValeur())
						{
							//la valeur est forcement celle contenu dans l'intervalle
							m1.setCase(i, j, inter.getMaxProba());
						}
						else
						{
							if(inter.IntervalleNul()||(som && sommeIntervalleFixe==0 && !inter.IntervalleUneValeur()))
							{
								//soit l'intervalle est nul, ou on est oblige de mettre 0 dans la case
								m1.setCase(i,j,0.0);
							}
							else
							{
								if(unintervalleaunevaleuregalea1&&inter.IntervalleUneValeur())
								{
									m1.setCase(i,j,1.0);
								}
								else
								{
									if(unintervalleaunevaleuregalea1&&!inter.IntervalleUneValeur())
									{
										m1.setCase(i, j, 0.0);
									}
									else{
										if(DernierIntervalleAyantPlusieursValeurs(matriceIntervalles, i, j))
										{
											//on cherche une valeur dans l'intervalle de telle sorte que la somme des valeurs fixes plus cette valeur soit egale a 1
											double val = ((double)(Math.round((1-SommeLigneSurLaligneSauflaCaseJ(m1, i, j, matriceIntervalles))*1000)))/1000;
											if((val<=(((double)(Math.round(inter.getMaxProba()*1000)))/1000)&&(val>=(((double)(Math.round(inter.getMinProba()*1000)))/1000))))
												m1.setCase(i,j,val);
											else
											{
												//pas de resultat trouve on recommence
												if(iter>100)
													return null;
												else
												{		
													for(int k=0;k<=j;k++)
														m1.setCase(i,k,null);
													iter++;
													j=-1;
												}
											}
										}
										else
										{
											if(DernierIntervalleNonNul(matriceIntervalles, i, j))
											{
												//idem
												double val = ((double)(Math.round((1-SommeLigneALaCaseJ(m1, i, j))*1000)))/1000;
												if((val<=(((double)(Math.round(inter.getMaxProba()*1000)))/1000)&&((val>=(((double)(Math.round(inter.getMinProba()*1000)))/1000)))))
													m1.setCase(i,j,val);
												else
												{
													//pas de resultat trouve on recommence
													if(iter>100)
														return null;
													else
													{		
														for(int k=0;k<=j;k++)
															m1.setCase(i,k,null);
														iter++;
														j=-1;
													}
												}
											}
											else
											{
												if((sommeIntervalleFixe!=0) && (!inter.IntervalleUneValeur()))
												{
													Random r= new Random();
													double m  = ((double)(r.nextInt((int)(Math.round((1-sommeIntervalleFixe)*1000))+1)))/1000;
													m1.setCase(i, j,m);
													sommeIntervalleFixe+=m;

												}
												else
												{
													float val=inter.RandomValeur();
													z++;
													while((!(ValeurValide(m1,(double)val/100,j!=matriceIntervalles.getNbColonne()-1,i,ValeurSuivanteNulle(j,i))))&&(z<=50))
													{
														val=inter.RandomValeur();
														z++;
													}

													if(z<=50)
													{	//la valeur trouve est bonne
														m1.setCase(i,j,(double)val/100);
													}
													else 
													{

														//pas de resultat trouve on recommence
														if(iter>100)
															return null;
														else
														{		
															for(int k=0;k<=j;k++)
																m1.setCase(i,k,null);
															iter++;
															j=-1;
														}
													}
												}
											}
										}
									}
								}
							}

						}
					}

				}
				else
					m1.setCase(i,j,0.0);

			}
		}
		return m1;
	}

	/**
	 * Determine si les valeurs après la colonne donné dans la ligne de la matrice
	 * sont nulles
	 * @param m Matrice<Intervalle>
	 * @param colonne
	 * @param ligne
	 * @return boolean
	 */
	public boolean ValeurSuivanteNulle(int colonne,int ligne)
	{
		boolean trouve=true;
		for(int j=colonne+1;j<matriceIntervalles.getNbColonne();j++)
		{
			if((matriceIntervalles.getCase(ligne,j)!=null)&&(matriceIntervalles.getCase(ligne,j).getMax()!=0))
				trouve=false;
		}
		return trouve;
	}

	/**
	 * Verifie s'il y a un seul intervalle dans la ligne qui soit non null ou
	 * egal à [0,0]
	 * pre-condition : le calcul est possible
	 * @param m Matrice<Intervalle>
	 * @param ligne int ligne a verifier
	 * @return boolean
	 */
	public boolean UnIntervalleDansLigne(Matrice<Intervalle>m,int ligne)
	{
		int nbinter=0;
		for (int i=0;i<matriceIntervalles.getNbColonne();i++)
		{
			if((matriceIntervalles.getCase(ligne,i)!=null)&&!matriceIntervalles.getCase(ligne,i).IntervalleNul())
				nbinter++;
		}
		return nbinter==1;
	}

	public boolean contientMatriceDouble(ArrayList<Matrice<Double>> liste, Matrice<Double> m)
	{
		for(int i=0; i<liste.size(); i++)
		{
			if(Matrice.CompareMatriceDouble(liste.get(i),m,EPSILON))
				return true;
		}
		return false;
	}



}
