/*
Copyright Bourdon Jeremie 
contributors : Giraudet Etienne, Docet Eric

jeremie.bourdon@univ-nantes.fr
eric.docet@etu.univ-nantes.fr
etienne.giraudet@etu.univ-nants.fr

This software is a computer program whose purpose is to do computations
on genetic graphs.

This software is governed by the CeCILL license under French law and
abiding by the rules of distribution of free software.  You can use, 
modify and/ or redistribute the software under the terms of the CeCILL
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL license and that you accept its terms.
*/

package cout;

import ihm.intervalle.Intervalle;
import java.io.Serializable;
/**
 * Classe representant l'ensemble des couts pouvant etre mis sur le graphe
 * @author giraudet-docet
 *
 */
public abstract class Cout implements Serializable
{
	private static final long serialVersionUID = 1L;
	protected String _nom;
	protected Intervalle _ideal;
	protected int duree;
	protected boolean activate;
	
	public Cout(String nom, Intervalle ideal, int n)
	{
		_nom = nom;
		_ideal = ideal;
		duree = n;
		activate = true;
	}
	
	
	public void setNom(String nom)
	{
		_nom=nom;
	}
	
	public void setActivation(boolean b)
	{
		activate = b;
	}
	public boolean getActivation()
	{
		return activate;
	}
	public String getNom()
	{
		return _nom;
	}
	
	public void setIdeal(Intervalle ideal)
	{
		_ideal=ideal;
	}
	
	public Intervalle getIdeal()
	{
		return _ideal;
	}
	
	public int getDuree()
	{
		return duree;
	}
	public void setDuree(int n)
	{
		duree = n;
	}
	
	public String toString()
	{
		return _nom;
	}
	
	//Test sur les types de cout
	public abstract boolean estCoutP1();
	public abstract boolean estCoutP2();
	public abstract boolean estCoutP3();
	public abstract boolean estCoutMult();
	

}
