/*
Copyright Bourdon Jeremie 
contributors : Giraudet Etienne, Docet Eric

jeremie.bourdon@univ-nantes.fr
eric.docet@etu.univ-nantes.fr
etienne.giraudet@etu.univ-nants.fr

This software is a computer program whose purpose is to do computations
on genetic graphs.

This software is governed by the CeCILL license under French law and
abiding by the rules of distribution of free software.  You can use, 
modify and/ or redistribute the software under the terms of the CeCILL
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL license and that you accept its terms.
*/

package dialog;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import ihm.Application;
import ihm.FenetrePrincipale;
import ihm.chemin.Chemin;
import ihm.dessin.DessinGrapheStateDefault;

import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.WindowConstants;

import util.JFloatField;
import util.JIntField;

/**
 * Dialog utilise lors d'un ajout de chemin dans le graphe
 * @author giraudet-docet
 *
 */
public class DialogAjoutChemin extends JDialog 
{
	private static final long serialVersionUID = 1L;
	
	public DialogAjoutChemin(final Chemin c, int n)
	{
		super(FenetrePrincipale.getInstance());
		JPanel pTout = new JPanel();
		add(pTout);
		
		addWindowListener(new WindowAdapter()
		{
			public void windowClosing(WindowEvent e){
				setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
				FenetrePrincipale.getInstance().getEspaceTravail().getDessinGraphe().setState(new DessinGrapheStateDefault(FenetrePrincipale.getInstance().getEspaceTravail().getDessinGraphe()));
				dispose();
			}
		});
		
		
		JLabel lbNom = new JLabel(Application.langue.getProperty("nom"));
		final JTextField tfNom = new JTextField(15);
		final JFloatField min = new JFloatField(10);
		final JFloatField max = new JFloatField(10);
		final JIntField dur = new JIntField(10);
		min.setFloat(0);
		max.setFloat(100);
		dur.setInt(1);
		JLabel ideal = new JLabel(Application.langue.getProperty("ideal"));
		JLabel _min = new JLabel(Application.langue.getProperty("mininter"));
		JLabel _max = new JLabel(Application.langue.getProperty("maxinter"));
		JLabel duree = new JLabel(Application.langue.getProperty("long"));
		tfNom.setText(Application.langue.getProperty("chemin")+n);
		c.setNom(Application.langue.getProperty("chemin")+n);
		JPanel pNom = new JPanel();
		pNom.add(lbNom);
		pNom.add(tfNom);
		JPanel ptext = new JPanel();
		ptext.add(ideal);
		JPanel pmin = new JPanel();
		pmin.add(_min);
		pmin.add(min);
		JPanel pmax = new JPanel();
		pmax.add(_max);
		pmax.add(max);
		JPanel pdur = new JPanel();
		pdur.add(duree);
		pdur.add(dur);
		pTout.setLayout(new BoxLayout(pTout,BoxLayout.Y_AXIS));
		pTout.add(pNom);
		pTout.add(ptext);
		pTout.add(pmin);
		pTout.add(pmax);
		pTout.add(pdur);
		
		JButton bValide = new JButton(Application.langue.getProperty("ok"));
		JPanel pBouton = new JPanel();
		pBouton.add(bValide);
		pTout.add(pBouton);
		bValide.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent e)
			{
				if(min.getFloat()>max.getFloat())
					new ErreurDialog(Application.langue.getProperty("erreurborne"));
				else
				{
					c.setNom(tfNom.getText());
					c.setIntervalle(min.getFloat(), max.getFloat());
					c.setDuree(dur.getInt());
					dispose();
				}
			}
		});
		
		setLocationRelativeTo(getParent());
		setResizable(false);
		setSize(350, 200);
		setVisible(true);
		
	}
	
	
}
