/*
Copyright Bourdon Jeremie 
contributors : Giraudet Etienne, Docet Eric

jeremie.bourdon@univ-nantes.fr
eric.docet@etu.univ-nantes.fr
etienne.giraudet@etu.univ-nants.fr

This software is a computer program whose purpose is to do computations
on genetic graphs.

This software is governed by the CeCILL license under French law and
abiding by the rules of distribution of free software.  You can use, 
modify and/ or redistribute the software under the terms of the CeCILL
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL license and that you accept its terms.
*/

package dialog;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import graphe.Graphe;
import ihm.Application;
import ihm.FenetrePrincipale;
import ihm.intervalle.Intervalle;
import ihm.transition.RelationProba;
import ihm.transition.Transition;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.ListSelectionModel;
import javax.swing.WindowConstants;

import util.ConvertArrayToList;
import util.JFloatField;

/**
 * Dialog servant a la creation ou la modification d'une relation sur les probabilite du graphe
 * @author giraudet-docet
 *
 */
public class DialogRelationProba extends JDialog 
{
	private static final long serialVersionUID = 1L;
	private Graphe graphe;
	private DialogRelationProba me=this; 
	private DialogListeRelationProba dlrp;
	private JList liste1;
	private JList liste2; 
	private JFloatField alpha;
	private JFloatField beta;
	private boolean modif;
	private RelationProba relation;
	
	public DialogRelationProba(DialogListeRelationProba dl, Graphe g)
	{
		super(FenetrePrincipale.getInstance());
		graphe = g;
		dlrp = dl;
		modif = false;
		initValeurDefaut();
		construitPanel();
		setTitle(Application.langue.getProperty("nomTitrerp"));
		setLocationRelativeTo(getParent());
		setSize(400, 300);
		setVisible(true);
		addWindowListener( new WindowAdapter(){
			public void windowClosing(WindowEvent e){
				setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
				new DialogListeRelationProba(graphe);
			}
		});
	}
	public DialogRelationProba(DialogListeRelationProba dl, RelationProba rp, Graphe g)
	{
		super(FenetrePrincipale.getInstance());
		relation = rp;
		graphe = g;
		dlrp = dl;
		modif = true;
		alpha = new JFloatField(4);
		beta = new JFloatField(4);
		alpha.setFloat(1f);
		beta.setFloat(0f);
		liste1 = new JList(ConvertArrayToList.convertListTransitions(graphe.listeTransitions));
		liste2 = new JList(ConvertArrayToList.convertListTransitions(graphe.listeTransitions));
		alpha.setFloat((float)rp.getAlpha());
		beta.setFloat((float)rp.getBeta());
		liste1.setSelectedValue(rp.getTransitionP1(), true);
		liste2.setSelectedValue(rp.getTransitionP2(), true);
		construitPanel();
		setTitle(Application.langue.getProperty("nomTitrerp"));
		setLocationRelativeTo(getParent());
		setSize(400, 300);
		setVisible(true);
		addWindowListener( new WindowAdapter(){
			public void windowClosing(WindowEvent e){
				setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
				new DialogListeRelationProba(graphe);
			}
		});
	}
	
	
	public void initValeurDefaut()
	{
		alpha = new JFloatField(4);
		beta = new JFloatField(4);
		alpha.setFloat(1f);
		beta.setFloat(0f);
		liste1 = new JList(ConvertArrayToList.convertListTransitions(graphe.listeTransitions));
		liste2 = new JList(ConvertArrayToList.convertListTransitions(graphe.listeTransitions));
		liste1.setSelectedIndex(0);
		liste2.setSelectedIndex(1);
	}
	
	public void construitPanel()
	{
		JPanel pTout = new JPanel();
		pTout.setBorder(BorderFactory.createTitledBorder("p1 = alpha*p2 + beta"));
		
		JPanel lp1 = new JPanel();
		lp1.setLayout(new BoxLayout(lp1,BoxLayout.Y_AXIS));
		lp1.add(new JLabel("p1"));
		liste1.setVisibleRowCount(10);
		liste1.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		JScrollPane sp1 = new JScrollPane(liste1);
		lp1.add(sp1);
		
		JPanel lp2 = new JPanel();
		lp2.setLayout(new BoxLayout(lp2,BoxLayout.Y_AXIS));
		lp2.add(new JLabel("p2"));
		liste2.setVisibleRowCount(10);
		liste2.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		JScrollPane sp2 = new JScrollPane(liste2);
		lp2.add(sp2);

		JPanel palpha = new JPanel();
		palpha.add(new JLabel("alpha="));
		palpha.add(alpha);
		
		JPanel pbeta = new JPanel();
		pbeta.add(new JLabel("beta="));
		pbeta.add(beta);
		
		JPanel value = new JPanel();
		value.setLayout(new BoxLayout(value,BoxLayout.Y_AXIS));
		value.add(palpha);
		value.add(pbeta);
		
		JButton bValide = new JButton(Application.langue.getProperty("ok"));
		value.add(bValide);
		bValide.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent e)
			{
				Transition p1 = (Transition)liste1.getSelectedValue();
				Transition p2 = (Transition)liste2.getSelectedValue();
				double a = alpha.getFloat();
				double b = beta.getFloat();
				if(relationCoherente(p1, p2, a, b))
				{
					if(p1==null && p2==null)
					{
						JOptionPane.showMessageDialog(me, Application.langue.getProperty("erreurRP1"),"ERROR", JOptionPane.ERROR_MESSAGE);
					}
					else
					{
						if(p1.equals(p2))
						{
							JOptionPane.showMessageDialog(me, Application.langue.getProperty("erreurRP2"),"ERROR", JOptionPane.ERROR_MESSAGE);
						}
						else
						{
							RelationProba rp = new RelationProba(p1,p2,alpha.getFloat(),beta.getFloat());
							p1.setRelation(rp);
							if(modif)
							{
								graphe.listeRelations.set(graphe.listeRelations.indexOf(relation), rp);
								new DialogListeRelationProba(graphe);
								dispose();
							}
							else
							{
								graphe.listeRelations.add(rp);
								dlrp.chargeListeRelation();
								new DialogListeRelationProba(graphe);
								dispose();
							}
						}
					}
				}
				else
				{
					JOptionPane.showMessageDialog(me, Application.langue.getProperty("erreurRP3"));
				}
			}
		});
		
		
		pTout.add(lp1);
		pTout.add(lp2);
		pTout.add(value);
		add(pTout);
	}
	
	public boolean relationCoherente(Transition p1, Transition p2, double alpha, double beta)
	{
		Intervalle i = p2.getIntervalleProbabilite();
		double valMinP1 = i.getMinProba()*alpha+beta;
		double valMaxP1 = i.getMaxProba()*alpha+beta;
		return (estProbabilite(valMinP1) && estProbabilite(valMaxP1));
	}
	
	public boolean estProbabilite(double p)
	{
		return (p>=0 && p<=1);
	}
	

	
	
}
