/*
Copyright Bourdon Jeremie 
contributors : Giraudet Etienne, Docet Eric

jeremie.bourdon@univ-nantes.fr
eric.docet@etu.univ-nantes.fr
etienne.giraudet@etu.univ-nants.fr

This software is a computer program whose purpose is to do computations
on genetic graphs.

This software is governed by the CeCILL license under French law and
abiding by the rules of distribution of free software.  You can use, 
modify and/ or redistribute the software under the terms of the CeCILL
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL license and that you accept its terms.
*/

package dialog;

import ihm.Application;
import ihm.chemin.Chemin;
import ihm.dessin.DessinGraphe;
import ihm.intervalle.Intervalle;
import thread.StartCalculVolume;
import thread.StartRechercheLocaleSimulation;
import util.JFloatField;
import util.JIntField;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.ArrayList;
import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;
import javax.swing.JSlider;
import javax.swing.JTextField;
import javax.swing.WindowConstants;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import cout.Cout;
import cout.CoutP1;
import calcul.Matrice;
import calcul.MethodeResolution;
import calcul.UsingMarks;
import graphe.Graphe;
/**
 * Classe qui permet de faire une simulation du calcul des coûts
 * @author Giraudet Docet
 *
 */
public class FenetreSimulation extends JFrame
{
	private static final long serialVersionUID = 1L;
	private Graphe _g;//graphe de la classe
	private Matrice<JSlider> ms;//matrice contenant les sliders
	private ArrayList<JFloatField> lf;//liste contenant les cases ou sont notes les resultats pour chaque cout
	private ArrayList<JFloatField> lfc;//liste contenant les cases ou sont notes les resultats pour chaque cout des chemins
	private ArrayList<JFloatField> lfmr;//liste contenant les cases ou sont places les methodes de resolution
	private ArrayList<JSlider> ljs;//liste contenant les sliders des couts
	private ArrayList<JSlider> ljsch;//liste contenant les sliders des couts des chemins
	private Matrice<Boolean> mb;//matrice qui contient des booleens pour savoir quel slider a ete bouge
	private Matrice<Boolean> mradio;//matrice qui contient des booleens pour svoir quel bouton radio ont ete coches
	private boolean calculcommence=false;//booleen pour savoir si le calcul a commence
	private ArrayList<Boolean> methodeActive;
	private Matrice<Intervalle> mi;
	protected static FenetreSimulation instance;

	/**
	 * Contstructeur de la classe qui prend en parametre un graphe
	 * @param g
	 */
	public FenetreSimulation(final DessinGraphe g)
	{
		super();
		addWindowListener(new WindowAdapter(){
			public void windowClosing(WindowEvent e){
				setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
				dispose();
			}
		});
		instance=this;
		this.setTitle(Application.langue.getProperty("nomTitrefs"));
		_g=g.graphe;
		mi =_g.construireMatriceIntervalle();
		mradio = new Matrice<Boolean>(mi.getNbLigne(),mi.getNbColonne());
		//initialisation de la matrice mradio
		for(int i=0;i<mradio.getNbLigne();i++)
			for(int j=0;j<mradio.getNbColonne();j++)
				mradio.setCase(i, j, false);
		//construction du panel contenant les sliders
		JPanel pSliders = new JPanel()
		{
			/**
			 * 
			 */
			private static final long serialVersionUID = 1L;

			public Dimension getMinimumSize()
			{
				return new Dimension(this.getWidth(),this.getHeight());
			}

			@Override
			public Dimension getPreferredSize()
			{
				return getMinimumSize();
			}

			@Override
			public Dimension getMaximumSize()
			{
				return getMinimumSize();
			}
		};
		int nbEtat=_g.getNombreEtats();
		pSliders.setLayout(new GridLayout(nbEtat,nbEtat));
		mb = new Matrice<Boolean>(nbEtat,nbEtat);
		AjoutSlider(pSliders,mi);
		//construction du panel contenant les boutons
		JPanel pBoutons = new JPanel();
		JButton start = new JButton(Application.langue.getProperty("bcm"));
		final JButton recherche = new JButton(Application.langue.getProperty("cas"));
		final JButton nbSol = new JButton(Application.langue.getProperty("vos"));
		//construction du panel contenant les couts
		JPanel pCout = new JPanel(){
			/**
			 * 
			 */
			private static final long serialVersionUID = 1L;

			public Dimension getMinimumSize()
			{
				return new Dimension(this.getWidth(),this.getHeight());
			}

			@Override
			public Dimension getPreferredSize()
			{
				return getMinimumSize();
			}

			@Override
			public Dimension getMaximumSize()
			{
				return getMinimumSize();
			}
		};
		JPanel Marks = new JPanel();
		Marks.setBorder(BorderFactory.createTitledBorder(Application.langue.getProperty("pCout")));
		AjoutCout(Marks,g.methodes);
		JPanel Path = new JPanel();
		Path.setBorder(BorderFactory.createTitledBorder(Application.langue.getProperty("pPath")));
		AjoutCoutChemin(Path,g.methodes);
		JPanel MR = new JPanel();
		MR.setBorder(BorderFactory.createTitledBorder(Application.langue.getProperty("pRequete")));
		AjoutMR(MR,g);
		pCout.setLayout(new BoxLayout(pCout,BoxLayout.Y_AXIS));
		pCout.add(Marks);
		pCout.add(Path);
		pCout.add(MR);
		//ajout du listener pour le bouton qui calcule les lamba' pour les valeurs de la mmatrice donne
		start.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e)
			{
				calculcommence=true;
				if(ModifSlide())
					Calcul(g.methodes);
				for(int i=0;i<mb.getNbLigne();i++)
					for(int j=0;j<mb.getNbColonne();j++)
					{
						mb.setCase(i, j, false);
						ms.getCase(i, j).setEnabled(true);
					}
				calculcommence=false;
			}
		});
		//ajout du listener pour le bouton qui calcule une solution pour des valeurs de la matrice fixee
		recherche.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e)
			{
				calculcommence=true;
				recherche.setEnabled(false);
				//on construit la matrice d'intervalle 
				Matrice<Intervalle> mIntervalle=new Matrice<Intervalle>(ms.getNbLigne(),ms.getNbColonne());
				for(int i=0;i<ms.getNbLigne();i++)
					for(int j=0;j<ms.getNbColonne();j++)
					{
						if(mradio.getCase(i, j))
						{
							double val = (double)(ms.getCase(i,j).getValue())/10;
							mIntervalle.setCase(i,j,new Intervalle(val,val));
						}
						else
							mIntervalle.setCase(i,j,mi.getCase(i, j));
					}
				//on lance la recherche locale
				ArrayList<MethodeResolution> lmr = new ArrayList<MethodeResolution>();
				lmr.add(new UsingMarks(_g));
				for(int i=0;i<methodeActive.size();i++)
					lmr.add(g.methodes.get(i+2));
				new StartRechercheLocaleSimulation(instance,mIntervalle,lmr,DessinGraphe.EPSILON,ms,mb,lf,lfc,lfmr,methodeActive,_g,recherche).start();
				calculcommence=false;
			}
		});
		nbSol.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent e)
			{
				final JDialog jd = new JDialog(instance,Application.langue.getProperty("vos"));
				final JIntField jif = new JIntField(10);
				JLabel jl = new JLabel(Application.langue.getProperty("nbpointsvolume"));
				jif.setInt(15);
				final JButton launch = new JButton(Application.langue.getProperty("lancer"));
				launch.addActionListener(new ActionListener()
				{
					public void actionPerformed(ActionEvent e)
					{
						jd.dispose();
						int nbp = jif.getInt();
						Matrice<Intervalle> mIntervalle=new Matrice<Intervalle>(ms.getNbLigne(),ms.getNbColonne());
						for(int i=0;i<ms.getNbLigne();i++)
							for(int j=0;j<ms.getNbColonne();j++)
							{
								
								if(mradio.getCase(i, j))
								{
									double val = (double)(ms.getCase(i,j).getValue())/10;
									mIntervalle.setCase(i,j,new Intervalle(val,val));
								}
								else
									mIntervalle.setCase(i,j,mi.getCase(i, j));
							}
						if(CalculPossible(mIntervalle))
						{
							int dim = Matrice.Dimension(mIntervalle);
							if(nbp>dim)
								new StartCalculVolume(mIntervalle,_g.getNombreEtats(),g.methodes,nbp,dim,true).start();
							else
								new ErreurDialog(Application.langue.getProperty("erreurvolume"));
						}
						else
							new ErreurDialog(Application.langue.getProperty("erreurFS1"));
						
						
					}
				});
				JButton cancel = new JButton(Application.langue.getProperty("annuler"));
				cancel.addActionListener(new ActionListener()
				{
					public void actionPerformed(ActionEvent e)
					{
						jd.dispose();
					}
				});
				JPanel jp = new JPanel();
				JPanel pBoutons = new JPanel();
				jp.add(jl);
				jp.add(jif);
				pBoutons.add(launch);
				pBoutons.add(cancel);
				jd.add(jp,BorderLayout.CENTER);
				jd.add(pBoutons,BorderLayout.SOUTH);
				jd.setLocationRelativeTo(instance);
				jd.setSize(300,125);
				jd.setVisible(true);
			}	
		});
		//construction de la fenetre
		pBoutons.add(start);
		pBoutons.add(recherche);
		pBoutons.add(nbSol);
		add(pBoutons,BorderLayout.SOUTH);
		JScrollPane jspSlider = new JScrollPane(pSliders);
		add(jspSlider,BorderLayout.CENTER);
		JScrollPane jspCout = new JScrollPane(pCout);
		add(jspCout,BorderLayout.EAST);
		Dimension tailleecran = getToolkit().getScreenSize();
		pSliders.setSize(nbEtat*275,nbEtat*80);
		pSliders.setMaximumSize(new Dimension(nbEtat*275,nbEtat*80));
		pCout.setSize(400,_g.getlisteMatriceCoutP1().size()*30);
		pCout.setMaximumSize(new Dimension(400,_g.getlisteMatriceCoutP1().size()*30));
		pBoutons.setSize(30,30);
		if(((pSliders.getSize().width+pCout.getSize().width+50)<tailleecran.width)&&((pSliders.getSize().height+pCout.getSize().height+pBoutons.getSize().height+50)<tailleecran.height))
			setSize(pSliders.getSize().width+pCout.getSize().width+50,pSliders.getSize().height+pCout.getSize().height+pBoutons.getSize().height+50);
		else 
			setSize(tailleecran);
		repaint();
		setVisible(true);
	}

	/**
	 * fonction qui ajoute les sliders au panel
	 * @param pSliders
	 * @param m
	 * @return
	 */
	private Matrice<JSlider> AjoutSlider(JPanel pSliders,Matrice<Intervalle> m)
	{
		ms = new Matrice<JSlider>(m.getNbLigne(),m.getNbColonne());
		for(int i=0;i<m.getNbLigne();i++)
		{
			for(int j=0;j<m.getNbColonne();j++)
			{
				JPanel p = new JPanel();
				p.setMaximumSize(new Dimension(300,50));
				p.setSize(300,50);
				p.setBorder(BorderFactory.createBevelBorder(1));//dessine un cadre
				Intervalle inter = m.getCase(i,j);
				final JSlider js = new JSlider(inter.getMin()*10,inter.getMax()*10);
				int ecart = (inter.getMin()*10)-(inter.getMax()*10);
				js.setPaintTicks(true);//dessine les traits de l'echelle
				js.setMinorTickSpacing(ecart/50);//espace minimum entre les petits traits des graduations
				js.setMajorTickSpacing(ecart/10);//espace maximum entre les grands traits des graduations
				//js.setPaintLabels(true);
				js.setMinimumSize(new Dimension(175,75));
				js.setPreferredSize(new Dimension(175,75));
				js.setMaximumSize(new Dimension(175,75));
				ms.setCase(i, j, js);
				mb.setCase(i, j, false);
				final JTextField Val = new JTextField(5);
				Float x = new Float(js.getValue())/1000;
				String s= x.toString();
				if(s.length()<=5)
					Val.setText(s);
				else
				{
					s=s.substring(0,5);
					Val.setText(s);
				}

				Val.setEditable(false);
				final int o=i;
				final int n=j;
				//ajout du listener au slider
				js.addChangeListener(new ChangeListener(){
					public void stateChanged(ChangeEvent e){
						{
							if(!calculcommence)
							{	//le calcul n'a pas commence 
								mb.setCase(o,n,true);//on enregistre que slider a ete bouge
								//on met la valeur dans la case
								String s= ((Float)(new Float(js.getValue())/1000)).toString();
								if(s.length()<=5)
									Val.setText(s);
								else
								{
									s=s.substring(0,5);
									Val.setText(s);
								}
								//on regarde s'il reste qu'un seul slider non bouge dans la ligne
								//si c'est le cas alors on doit bloquer le dernier slider
								int nbsliderNonBouge=0;
								JSlider jsp=null;
								for(int m=0;m<mb.getNbColonne();m++)
								{
									if(!mb.getCase(o, m))
									{
										nbsliderNonBouge++;
										jsp=ms.getCase(o,m);
									}
								}

								if(nbsliderNonBouge==1)
									jsp.setEnabled(false);
							}


							else
							{//le calcul a commence donc on reactualise la valeur dans la cas
								String s= ((Float)(new Float(js.getValue())/1000)).toString();
								if(s.length()<=5)
									Val.setText(s);
								else
								{
									s=s.substring(0,5);
									Val.setText(s);
								}
							}
						}
					}
				});
				//on ajoute les boutons radios
				final JRadioButton jr = new JRadioButton();
				jr.addActionListener(new ActionListener()
				{
					public void actionPerformed(ActionEvent e)
					{
						mradio.setCase(o, n, jr.isSelected())	;				
					}
				});
				JLabel jl = new JLabel("transition "+_g.listeEtats.get(i).getnum()+" -> "+_g.listeEtats.get(j).getnum());
				JPanel plabel = new JPanel();
				plabel.add(jl);
				JPanel pSlide = new JPanel();
				pSlide.add(jr);
				pSlide.add(js);
				pSlide.add(Val);
				p.add(jl);
				p.add(pSlide);
				pSliders.add(p);
			}
		}
		return ms;

	}

	/**
	 * fonction qui construit le panel contenant les resultats pour les couts du graphe
	 * @param pCout
	 * @return
	 */
	private boolean sliderBouge = false;
	private void AjoutCout(JPanel pCout,final ArrayList<MethodeResolution> mr)
	{

		lf= new ArrayList<JFloatField>();
		ljs = new ArrayList<JSlider>();
		ArrayList<CoutP1> lc = _g.getlisteMatriceCoutP1();
		pCout.setLayout(new GridLayout(lc.size(),1));
		for(int i=0;i<lc.size();i++)
		{
			final Cout c = lc.get(i);
			JLabel nc = new JLabel(c.getNom());
			final JFloatField jf= new JFloatField(6);
			final JFloatField jfc= new JFloatField(6);
			if(c.getIdeal().moyenne().toString().length()<6)
				jfc.setText(c.getIdeal().moyenne().toString());
			else
				jfc.setText(c.getIdeal().moyenne().toString().substring(0, 6));
			lf.add(jf);
			final JSlider js = new JSlider((int)((c.getIdeal().getMinDouble()).floatValue()*10000),(int)((c.getIdeal().getMaxDouble()).floatValue()*10000));
			final JCheckBox checkActivation = new JCheckBox();
			checkActivation.setSelected(c.getActivation());
			int ecart = (int)((c.getIdeal().getMinDouble()).floatValue()*10000)-(int)((c.getIdeal().getMaxDouble()).floatValue()*10000);
			js.setPaintTicks(true);//dessine les traits de l'echelle
			js.setMinorTickSpacing(ecart/50);//espace minimum entre les petits traits des graduations
			js.setMajorTickSpacing(ecart/10);//espace maximum entre les grands traits des graduations
			ljs.add(js);
			js.setMaximum((int)((c.getIdeal().getMaxDouble()).floatValue()*10000));
			js.setMinimum((int)((c.getIdeal().getMinDouble()).floatValue()*10000));
			js.setMinimumSize(new Dimension(75,75));
			js.setPreferredSize(new Dimension(75,75));
			js.setMaximumSize(new Dimension(75,75));
			js.addMouseListener(new MouseListener()
			{
				public void mouseClicked(MouseEvent e) {
				}
				public void mouseEntered(MouseEvent e) {
				}
				public void mouseExited(MouseEvent e) {
				}
				public void mousePressed(MouseEvent e) {
				}
				public void mouseReleased(MouseEvent e) {
					double val = new Double(js.getValue())/10000;
					if(sliderBouge)
					{
						jfc.setText(new Double(val).toString());
						sliderBouge = false;
						CalculCout(mr);
						for(int i=0;i<mb.getNbLigne();i++)
							for(int j=0;j<mb.getNbColonne();j++)
							{
								mb.setCase(i, j, false);
								ms.getCase(i, j).setEnabled(true);
							}
					}
				}
			});
			js.addChangeListener(new ChangeListener()
			{
				public void stateChanged(ChangeEvent e) 
				{
					sliderBouge = true;
					jfc.setText(new Double(new Double(js.getValue())/10000).toString());
				}
				
			});
			checkActivation.addActionListener(new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					c.setActivation(checkActivation.isSelected());
					js.setEnabled(!checkActivation.isSelected());
					if(c.getActivation())
					{
						jf.setBackground(Color.WHITE);
					}
					else
						jf.setBackground(Color.GRAY);
				}
			});
			JPanel pc = new JPanel();
			pc.setBorder(BorderFactory.createBevelBorder(1));
			pc.add(nc);
			pc.add(jf);
			pc.add(js);
			pc.add(jfc);
			pc.add(checkActivation);
			pc.setSize(new Dimension(400,30));
			pCout.add(pc);

		}

	}



	/**
	 * pre-cond : il faut d'abord appeler la fonction AjoutCout
	 * fonction qui construit le panel contenant les resultats pour les couts des chemins des graphes
	 * @param pCout
	 * @return
	 */
	private void AjoutCoutChemin(JPanel pCout,final ArrayList<MethodeResolution> mr)
	{
		pCout.setLayout(new GridLayout(_g.getChemins().size(),1));
		lfc= new ArrayList<JFloatField>();
		ljsch = new ArrayList<JSlider>();
		final ArrayList<Chemin> lc = _g.getChemins();
		for(int i=0;i<lc.size();i++)
		{
			final int t = i;
			final Cout c = lc.get(i).getCout();
			JLabel nc = new JLabel(c.getNom());
			final JFloatField jf= new JFloatField(6);
			final JFloatField jfc= new JFloatField(6);
			jfc.setText(c.getIdeal().moyenne().toString());
			lfc.add(jf);
			final JSlider js = new JSlider((int)((c.getIdeal().getMinDouble()).floatValue()*10000),(int)((c.getIdeal().getMaxDouble()).floatValue()*10000));
			final JCheckBox checkActivation = new JCheckBox();
			checkActivation.setSelected(c.getActivation());
			int ecart = (int)((c.getIdeal().getMinDouble()).floatValue()*10000)-(int)((c.getIdeal().getMaxDouble()).floatValue()*10000);
			js.setPaintTicks(true);//dessine les traits de l'echelle
			js.setMinorTickSpacing(ecart/50);//espace minimum entre les petits traits des graduations
			js.setMajorTickSpacing(ecart/10);//espace maximum entre les grands traits des graduations
			ljsch.add(js);
			js.setMaximum((int)(((c.getIdeal().getMaxDouble()).floatValue()*10000)));
			js.setMinimum((int)(((c.getIdeal().getMinDouble()).floatValue()*10000)));
			js.setMinimumSize(new Dimension(75,75));
			js.setPreferredSize(new Dimension(75,75));
			js.setMaximumSize(new Dimension(75,75));
			js.addMouseListener(new MouseListener()
			{
				public void mouseClicked(MouseEvent e) {
				}
				public void mouseEntered(MouseEvent e) {
				}
				public void mouseExited(MouseEvent e) {
				}
				public void mousePressed(MouseEvent e) {
				}
				public void mouseReleased(MouseEvent e) {
					double val = new Double(js.getValue())/10000;
					if(val != jfc.getFloat())
					{
						jfc.setText(new Double(val).toString());
						CalculCout(mr);
						for(int i=0;i<mb.getNbLigne();i++)
							for(int j=0;j<mb.getNbColonne();j++)
							{
								mb.setCase(i, j, false);
								ms.getCase(i, j).setEnabled(true);
							}
					}
				}
			});
			js.addChangeListener(new ChangeListener()
			{
				public void stateChanged(ChangeEvent e) 
				{
					jfc.setText(new Double(new Double(js.getValue())/10000).toString());
				}
				
			});
			checkActivation.addActionListener(new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					c.setActivation(checkActivation.isSelected());
					lc.get(t).setActivation(checkActivation.isSelected());
					js.setEnabled(!checkActivation.isSelected());
					if(c.getActivation())
						jf.setBackground(Color.WHITE);
					else
						jf.setBackground(Color.GRAY);
				}
			});
			JPanel pc = new JPanel();
			pc.setBorder(BorderFactory.createBevelBorder(1));
			pc.add(nc);
			pc.add(jf);
			pc.add(js);
			pc.add(jfc);
			pc.add(checkActivation);
			pc.setSize(new Dimension(400,30));
			pCout.add(pc);
		}

	}

	/**
	 * Ajoute les methodes de resolution sur le JPanel MR
	 * @param MR
	 */
	public void AjoutMR(JPanel MR,DessinGraphe dg)
	{
		MR.setLayout(new GridLayout(dg.methodes.size()-1,1));
		methodeActive = new ArrayList<Boolean>();
		lfmr = new ArrayList<JFloatField>();
		for(int i=2;i<dg.methodes.size();i++)
		{
			final int t =i;
			JPanel truc = new JPanel();
			JLabel jl= new JLabel(dg.methodes.get(i).toString());
			final JCheckBox jcb = new JCheckBox();
			final JFloatField jtf = new JFloatField(6);
			methodeActive.add(false);
			jcb.addActionListener(new ActionListener(){
				public void actionPerformed(ActionEvent e)
				{
					methodeActive.set(t-2,jcb.isSelected());
					jtf.setEnabled(jcb.isSelected());
					jtf.setBackground(Color.WHITE);
				}
			});
			jcb.addActionListener(new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					jtf.setEnabled(jcb.isSelected());
					if(jcb.isSelected())
						jtf.setBackground(Color.WHITE);
					else
						jtf.setBackground(Color.GRAY);
				}
			});
			jtf.setEnabled(jcb.isSelected());
			jtf.setBackground(Color.GRAY);
			lfmr.add(jtf);
			truc.add(jcb);
			truc.add(jl);
			truc.add(jtf);
			MR.add(truc);
		}
	}


	/**
	 * fonction qui calcule les couts quand on bouge les sliders des couts
	 *
	 */
	public void CalculCout(ArrayList<MethodeResolution> methodes)
	{
		calculcommence=true;
//		on construit la matrice d'intervalle 
		Matrice<Intervalle> mIntervalle=new Matrice<Intervalle>(ms.getNbLigne(),ms.getNbColonne());
		for(int i=0;i<ms.getNbLigne();i++)
			for(int j=0;j<ms.getNbColonne();j++)
			{
				if(mradio.getCase(i, j))
				{
					double val = (double)(ms.getCase(i,j).getValue())/10;
					mIntervalle.setCase(i,j,new Intervalle(val,val));
				}
				else
					mIntervalle.setCase(i,j,mi.getCase(i, j));
			}
		//on construit la liste des k
		ArrayList<Double> k = new ArrayList<Double>();
		for(int i=0;i<ljs.size();i++)
			k.add((double)(ljs.get(i).getValue())/10000);
		for(int i=0;i<ljsch.size();i++)
			k.add((double)(ljsch.get(i).getValue())/10000);
		//on lance la recherche locale
		ArrayList<MethodeResolution> lmr = new ArrayList<MethodeResolution>();
		lmr.add(new UsingMarks(_g,k));
		for(int i=0;i<methodeActive.size();i++)
			lmr.add(methodes.get(i+2));
		new StartRechercheLocaleSimulation(instance,_g.construireMatriceIntervalle(), lmr, DessinGraphe.EPSILON,ms,mb,lf,lfc,lfmr,methodeActive,_g).start();

		calculcommence=false;
	}

	/**
	 * fonction qui calcul les lambda' dans ou on clique sur le bouton calculate one solution
	 *
	 */
	public void Calcul(ArrayList<MethodeResolution> methodes)
	{
		//construction de la matrice de double correspondant au valeur des sliders
		Matrice<Intervalle> mi = new Matrice<Intervalle>(ms.getNbLigne(),ms.getNbColonne());
		for(int i=0;i<ms.getNbLigne();i++)
		{
			for(int j=0;j<ms.getNbColonne();j++)
			{
				JSlider js = ms.getCase(i,j);
				Intervalle val = new Intervalle(((double)js.getValue())/10,((double)js.getValue())/10);
				mi.setCase(i, j,val);
			}
		}
		ArrayList<MethodeResolution> lmr = new ArrayList<MethodeResolution>();
		lmr.add(new UsingMarks(_g));
		for(int i=0;i<methodeActive.size();i++)
			lmr.add(methodes.get(i+2));
		new StartRechercheLocaleSimulation(instance,mi, lmr, DessinGraphe.EPSILON,ms,mb,lf,lfc,lfmr,methodeActive,_g).start();
	}

	/**
	 * fonction qui permet d'ajuster les valeurs des slides avant d'effectuer le calcul
	 * de telle sorte que la somme de chaque ligne de la matrice soit egal à 1
	 * @return
	 */
	private boolean ModifSlide()
	{
		Matrice<Intervalle> mi = _g.construireMatriceIntervalle();
		//on parcourt la liste des sliders et on bouge seulement les sliders qui n'ont pas ete deplace
		//si on peut pas repercute le decalage alors on affiche une erreur 
		for(int i=0;i<ms.getNbLigne();i++)
		{
			int x=SommeLigne(i);
			if(x!=1000)
			{
				Intervalle inter = null;
				//on compte le nombre de sliders non bouge
				int nbSliderNonBouge=0;
				for(int j=0;j<mb.getNbColonne();j++)
				{
					inter = mi.getCase(i, j);
					if(!mb.getCase(i,j)&&!inter.IntervalleUneValeur())
						nbSliderNonBouge++;
				}
				for(int j=0;j<ms.getNbColonne();j++)
				{
					inter = mi.getCase(i, j);
					if(nbSliderNonBouge!=0&&!inter.IntervalleUneValeur())
					{
						//on compte le decalage
						int decalage=-((1000-x)/nbSliderNonBouge);
						if(!mb.getCase(i, j))
						{
							JSlider js = ms.getCase(i,j);
							int val=js.getValue();
							if(((val-decalage)<=(inter.getMax()*10))&&((val-decalage)>=(inter.getMin()*10)))
							{
								//Le decalage est inclus dans l'intervalle
								js.setValue(val-decalage);
								x=x-decalage;
							}
							else
								if((val-decalage)>(inter.getMax()*10))
								{
									//le decalage fait depasser la borne max
									x=x+(inter.getMax()*10-val);
									js.setValue(inter.getMax()*10);
								}
								else
								{
									//le decalage fait depasser la borne min
									x=x-(inter.getMin()*10-val);
									js.setValue(inter.getMin()*10);
								}
							nbSliderNonBouge--;
						}
					}
				}
				if(x!=1000)
				{
					new ErreurDialog(Application.langue.getProperty("erreurFS2"));
					return false;
				}

			}
		}

		return true;
	}

	/**
	 * fonction qui la somme sur une ligne de la matrice de sliders
	 * @param i
	 * @return
	 */
	private int SommeLigne(int i)
	{
		int som = 0;
		for(int j=0;j<ms.getNbColonne();j++)
		{
			som+=ms.getCase(i, j).getValue();
		}
		return som;
	}
	
	public static boolean CalculPossible(Matrice<Intervalle> matriceIntervalles)
	{
		Intervalle inter;
		for(int i=0;i<matriceIntervalles.getNbLigne();i++)
		{
			inter=new Intervalle(0,0);
			for(int j=0;j<matriceIntervalles.getNbColonne();j++)
			{
				if(matriceIntervalles.getCase(i,j)!=null)
					inter = inter.somme(matriceIntervalles.getCase(i,j));
			}
			if((Math.round(inter.getMinDouble())>100)||(Math.round(inter.getMaxDouble())<100))
			{
				return false;
			}
		}
		return true;
	}
	
	
}
