/*
Copyright Bourdon Jeremie 
contributors : Giraudet Etienne, Docet Eric

jeremie.bourdon@univ-nantes.fr
eric.docet@etu.univ-nantes.fr
etienne.giraudet@etu.univ-nants.fr

This software is a computer program whose purpose is to do computations
on genetic graphs.

This software is governed by the CeCILL license under French law and
abiding by the rules of distribution of free software.  You can use, 
modify and/ or redistribute the software under the terms of the CeCILL
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL license and that you accept its terms.
*/

package ihm;

import ihm.dessin.DessinGraphe;

import java.awt.BorderLayout;
import java.awt.Color;

import javax.swing.*;

import org.sbml.libsbml.Model;

/**
 * Classe qui herite de la classe JPanel et qui permet d'afficher le 
 * dessin du graphe et le menu associee
 * @author Docet Giraudet
 *
 */
public class EspaceTravail extends JPanel 
{
	static final long serialVersionUID = 0;
	protected JLabel lbBarreEtat = new JLabel();
	protected transient JScrollPane jsp;
	protected transient JScrollPane jspmenu;
	protected DessinGraphe dessG;
	protected MenuTraitement menuT;
	
	/**
	 * Methode qui permet de lui donner l'instance de la classe DessinGraphe
	 * @param dg
	 */
	public void setDessG(DessinGraphe dg)
	{
		dessG=dg;
		menuT=new MenuTraitement(dessG);
		dessG.setMenu(menuT);
		setLayout(new BorderLayout());


		add(dessG,BorderLayout.CENTER);
		add(menuT,BorderLayout.EAST);

		//ajout du Scroll sur le dessin
		jsp = new JScrollPane(dessG);
		add(jsp,BorderLayout.CENTER);
		
		jspmenu = new JScrollPane(menuT);
		add(jspmenu,BorderLayout.EAST);
		
		lbBarreEtat.setText(Application.langue.getProperty("lbBarreEtat"));
		add(lbBarreEtat,BorderLayout.SOUTH);
	}
	
	/**
	 * Methode qui permet de sauvegarder un fichier SBML
	 * @param m Model
	 */
	public void SauvegarderSBMLFile(Model m)
	{
		dessG.SauvegarderSBMLFile(m);
	}
	
	/**
	 * Constructeur vide de la classe 
	 * 
	 */
	public EspaceTravail()
	{	
	}
	
	/**
	 * Fonction qui renvoie le menu de traitement
	 */
	public MenuTraitement getMenuTraitement()
	{
		return menuT;
	}
	
	public DessinGraphe getDessinGraphe()
	{
		return dessG;
	}
	
	/**
	 * Methode qui supprime tous les composant ajouter au panel
	 *
	 */
	public void supprimeAllComposant()
	{
		this.removeAll();
	}
	

	
	public void setResultatGraphe(boolean x)
	{
		dessG.setResultatGraphe(x);
	}
	
	public void setResultatFile(boolean b)
	{
		dessG.setResultatFile(b);
	}
	
	public void setBarreEtat(String s)
	{
		lbBarreEtat.setText(s);
	}
	public void setBarreEtatDefaut()
	{
		lbBarreEtat.setText(Application.langue.getProperty("lbBarreEtat"));
	}
	public void reorganiserGrappa()
	{
		dessG.reorganiserGrappa();
	}
	public void setRayon(int t)
	{
		dessG.setRayon(t);
	}
	public void setColorEtat(Color c)
	{
		dessG.setColorEtat(c);
	}
	public void setColorTransition(Color c)
	{
		dessG.setColorTransition(c);
	}
	
}
