/*
Copyright Bourdon Jeremie 
contributors : Giraudet Etienne, Docet Eric

jeremie.bourdon@univ-nantes.fr
eric.docet@etu.univ-nantes.fr
etienne.giraudet@etu.univ-nants.fr

This software is a computer program whose purpose is to do computations
on genetic graphs.

This software is governed by the CeCILL license under French law and
abiding by the rules of distribution of free software.  You can use, 
modify and/ or redistribute the software under the terms of the CeCILL
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL license and that you accept its terms.
*/

package ihm;

import graphe.Graphe;
import ihm.dessin.DessinGraphe;
import ihm.intervalle.Intervalle;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Graphics;
import java.awt.PrintJob;
import java.awt.Toolkit;
import java.awt.event.*;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.ArrayList;
import java.util.Properties;
import javax.swing.*;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import org.sbml.libsbml.*;

import calcul.Matrice;
import calcul.RechercheLocale;
import thread.StartCalculVolume;
import thread.StartCoherenceBornes;
import thread.ThreadCalcul;
import thread.StartCalculCourbeGraph;
import util.JIntField;
import dialog.APropos;
import dialog.DialogArriveeEtat;
import dialog.DialogConfirmeQuitter;
import dialog.DialogEnregistrement;
import dialog.DialogListeRelationProba;
import dialog.DialogParametreRL;
import dialog.DialogProbaNePasPasser;
import dialog.DialogUneFoisParUnChemin;
import dialog.DialogUneFoisParUnEtat;
import dialog.ErreurDialog;
import dialog.FenetreSimulation;
/**
 * Classe qui hérite de la classe JFrame, et où on crée la barre de menu 
 * et une instance de la classe EspaceTravail
 * @author Docet Giraudet
 *
 */
public class FenetrePrincipale extends JFrame
{
	static final boolean ouvreVide = true;
	protected static FenetrePrincipale instance = null;
	public static final String NOM_LOGICIEL = "POGG";
	static final long serialVersionUID = 0;
	private EspaceTravail et;
	private File f;
	private String pathFileResult;
	private boolean dessinOuvert;
	private ArrayList<ThreadCalcul> ltc;
	public static Properties config;

	/**
	 * Constructeur de la classe, qui cree la barre de menu et 
	 * l'instance de la classe EspaceTravail
	 */
	protected FenetrePrincipale()
	{
		//définition de la taille de base de la fentre
		setSize(Toolkit.getDefaultToolkit().getScreenSize());
		NomMenu();
		//ajout des menus
		initMenus();
		//ajout du titre
		setTitle(NOM_LOGICIEL);
		dessinOuvert=false;
		f=null;
		et = new EspaceTravail();
		ltc = new ArrayList<ThreadCalcul>();
		getContentPane().add(et);
		

		//Listener window : fermeture de la fenetre 
		addWindowListener( new WindowAdapter(){
			public void windowClosing(WindowEvent e){
				setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
				if(dessinOuvert)
				{
					quitter_enregistrer(true);
				}
				else 
				{
					fermerApplication(true);
				}
			}
		});
	}

	public void retireThreadCalcul(ThreadCalcul tc)
	{
		ltc.remove(tc);
	}

	public ArrayList<ThreadCalcul> getLTC()
	{
		return ltc;
	}

	/**
	 * Cette fonction permet de n'instancier qu'une seule fenetre principale.
	 * @return l'unique FenetrePrincipale.
	 */
	public static FenetrePrincipale getInstance()
	{
		if (instance == null)
		{
			instance = new FenetrePrincipale();
		}
		return instance;
	}

	/**
	 * Cette fonction renvoie l'espace de travail
	 */
	public EspaceTravail getEspaceTravail()
	{
		return et;
	}
	
	/**
	 * Cette methode ferme la partie dessin et le menu si rattachant
	 */
	public void fermerDessin()
	{
		et.supprimeAllComposant();
		setTitle(NOM_LOGICIEL);
		activationMenus(false);
		repaint();
	}

	/**
	 * Cette methode permet de quitter en enregistrant.
	 * Cette methode lance une DialogEnregistrement
	 * Le boolean x, si x est à true alors on ferme l'application sinon on ferme 
	 * juste le dessin
	 * @param x
	 */
	public void quitter_enregistrer(final boolean x)
	{
		JButton oui = new JButton(Application.langue.getProperty("oui"));
		JButton non = new JButton(Application.langue.getProperty("non"));
		JButton annuler = new JButton(Application.langue.getProperty("annuler"));
		final DialogEnregistrement j = new DialogEnregistrement(oui,non,annuler);
		oui.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent e)
			{
				j.dispose();
				if(f==null)
				{
					f=getFile(true);
				}
				if(f!=null)
				{
					SauvegardeFichier(f);
				}

				if(x)
					fermerApplication(false);
				else
					fermerDessin();
			}
		});
		non.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent e)
			{
				j.dispose();
				if(x)
					fermerApplication(false);
				else
					fermerDessin();
			}
		});
		annuler.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent e)
			{
			}
		});
	}



	/**
	 * item du menu
	 */
	private JMenu menuFichier; 
	private JMenuItem nouveau;
	private JMenuItem ouvrir;
	private JMenuItem enregistrer; 
	private JMenuItem enregistrer_sous; 
	private JMenuItem imprimer ;
	private JMenuItem quitter ;
	private JMenuItem fermer ;
	private JMenuItem importer; 
	private JMenuItem exporter;
	private JMenuItem importeTexte; 
	private JMenuItem importDot ;
	private JMenuItem exportDot ;
	
	private JMenu menuCalculs;
	private JMenu requetes;
	private JMenuItem arriveeEtat;
	private JMenuItem arriveeChemin;
	private JMenuItem coherenceBornes;
	private JMenuItem ensembleSolution;
	private JMenuItem uneFoisParUnEtat; 
	private JMenuItem uneFoisParUnChemin; 
	private JMenuItem auMoinsUneFoisParUnEtat; 
	private JMenuItem auMoinsUneFoisParUnChemin;
	private JMenuItem aucuneFoisParUnEtat; 
	private JMenuItem aucuneFoisParUnChemin; 
	private JMenuItem courbe;
	
	private JMenu menuOptions;
	private JMenu menuAffichage;
	private JCheckBoxMenuItem resGraphe;
	private JMenuItem organiserGraphe;
	private JMenuItem organiserGrappa;
	private JCheckBoxMenuItem resFile;
	private JMenuItem fileDirectory;
	private JMenuItem contrainteProba;
	private JMenuItem simulation;
	private JMenuItem tailleEtat;
	private JMenuItem couleurEtat;
	private JMenuItem couleurTransition;
	private JMenu avance;
	private JMenuItem paramRl;
	private JMenuItem paramDef;
	private JMenuItem sauvParam;
	
	private JMenu aide;
	private JMenuItem apropos;
	
	/**
	 * fonction qui donne le nom a chaque item dans le menu de la fenetre
	 *
	 */
	public void NomMenu()
	{
		 menuFichier = new JMenu(Application.langue.getProperty("fichier"));
		nouveau = new JMenuItem(Application.langue.getProperty("nouveau"));
		 ouvrir = new JMenuItem(Application.langue.getProperty("ouvrir"));
		 enregistrer = new JMenuItem(Application.langue.getProperty("enregistrer"));
		enregistrer_sous = new JMenuItem(Application.langue.getProperty("enregistrer_sous"));
		imprimer = new JMenuItem(Application.langue.getProperty("imprimer"));
		 quitter = new JMenuItem(Application.langue.getProperty("quitter"));
		 fermer = new JMenuItem(Application.langue.getProperty("fermer"));
		 importer = new JMenuItem(Application.langue.getProperty("importer"));
		 exporter = new JMenuItem(Application.langue.getProperty("exporter"));
		importeTexte = new JMenuItem(Application.langue.getProperty("importeTexte"));
		importDot = new JMenuItem(Application.langue.getProperty("importDot"));
		exportDot = new JMenuItem(Application.langue.getProperty("exportDot"));
		
		 menuCalculs = new JMenu(Application.langue.getProperty("menuCalculs"));
		 requetes = new JMenu(Application.langue.getProperty("requetes"));
		 arriveeEtat = new JMenuItem(Application.langue.getProperty("arriveeEtat"));
		 arriveeChemin = new JMenuItem(Application.langue.getProperty("arriveeChemin"));
		coherenceBornes = new JMenuItem(Application.langue.getProperty("coherenceBornes"));
		 ensembleSolution = new JMenuItem(Application.langue.getProperty("ensembleSolution"));
		  courbe = new JMenuItem(Application.langue.getProperty("courbe"));
		  
		  
		 uneFoisParUnEtat = new JMenuItem(Application.langue.getProperty("uneFoisParUnEtat")); 
		 uneFoisParUnChemin = new JMenuItem(Application.langue.getProperty("uneFoisParUnChemin")); 
		  auMoinsUneFoisParUnEtat = new JMenuItem(Application.langue.getProperty("auMoinsUneFoisParUnEtat")); 
		  auMoinsUneFoisParUnChemin = new JMenuItem(Application.langue.getProperty("auMoinsUneFoisParUnChemin"));
		  aucuneFoisParUnEtat = new JMenuItem(Application.langue.getProperty("aucuneFoisParUnEtat")); 
		  aucuneFoisParUnChemin = new JMenuItem(Application.langue.getProperty("aucuneFoisParUnChemin"));
		  
		
		  menuOptions = new JMenu(Application.langue.getProperty("menuOptions"));
		 menuAffichage = new JMenu(Application.langue.getProperty("menuAffichage"));
		 resGraphe = new JCheckBoxMenuItem(Application.langue.getProperty("resGraphe"));
		  organiserGraphe = new JMenuItem(Application.langue.getProperty("organiserGraphe"));
		  organiserGrappa = new JMenuItem(Application.langue.getProperty("organiserGrappa"));
		 resFile = new JCheckBoxMenuItem(Application.langue.getProperty("resFile"));
		  fileDirectory = new JMenuItem(Application.langue.getProperty("fileDirectory"));
		  contrainteProba = new JMenuItem(Application.langue.getProperty("contrainteProba"));
		 simulation = new JMenuItem(Application.langue.getProperty("simulation"));
		  tailleEtat = new JMenuItem(Application.langue.getProperty("tailleEtat"));
		  couleurEtat = new JMenuItem(Application.langue.getProperty("couleurEtat"));
		  couleurTransition = new JMenuItem(Application.langue.getProperty("couleurTransition"));
		  avance=new JMenu(Application.langue.getProperty("avance"));
		  paramRl=new JMenuItem(Application.langue.getProperty("paramRl"));
		  paramDef=new JMenuItem(Application.langue.getProperty("paramDef"));
		  sauvParam=new JMenuItem(Application.langue.getProperty("sauvParam"));
		
		 aide = new JMenu(Application.langue.getProperty("aide"));
		  apropos = new JMenuItem(Application.langue.getProperty("apropos")+" "+NOM_LOGICIEL);
	}
	
	/**
	 * fonction qui creer le menu et qui ajout tous les listeners
	 *
	 */
	public void initMenus()
	{
		nouveau.setAccelerator(KeyStroke.getKeyStroke(78,java.awt.Event.CTRL_MASK));
		ouvrir.setAccelerator(KeyStroke.getKeyStroke(79,java.awt.Event.CTRL_MASK));
		imprimer.setAccelerator(KeyStroke.getKeyStroke(73,java.awt.Event.CTRL_MASK));
		quitter.setAccelerator(KeyStroke.getKeyStroke(81,java.awt.Event.CTRL_MASK));
		fermer.setAccelerator(KeyStroke.getKeyStroke(87,java.awt.Event.CTRL_MASK));
		enregistrer.setAccelerator(KeyStroke.getKeyStroke(83,java.awt.Event.CTRL_MASK));
		//on ne peut enregistrer que si on a ouvert un dessin
		activationMenus(false);
		resFile.setEnabled(false);

		//code pour le menu item importer, ou l'on utilise une instance de la classe DialogEnregistrement
		importer.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent e)
			{
				if(dessinOuvert)
				{
					JButton oui = new JButton(Application.langue.getProperty("oui"));
					JButton non = new JButton(Application.langue.getProperty("non"));
					JButton annuler = new JButton(Application.langue.getProperty("annuler"));
					final DialogEnregistrement j =new DialogEnregistrement(oui,non,annuler);
					oui.addActionListener(new ActionListener(){
						public void actionPerformed(ActionEvent e)
						{
							j.dispose();
							if(f==null)
							{
								f=getFile(true);
							}
							if(f!=null)
							{
								SauvegardeFichier(f);
							}

							Importer();
						}
					});
					non.addActionListener(new ActionListener()
					{
						public void actionPerformed(ActionEvent e)
						{
							j.dispose();
							fermerDessin();

							Importer();
						}
					});
					annuler.addActionListener(new ActionListener(){
						public void actionPerformed(ActionEvent e)
						{

						}
					});
				}
				else
				{
					Importer();
				}
			}
		});

		//code pour le menu item exporter
		exporter.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e)
			{
				Exporter();
			}
		});

		//code pour le menu item importerTexte
		importeTexte.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e)
			{
				if(dessinOuvert)
				{
					JButton oui = new JButton(Application.langue.getProperty("oui"));
					JButton non = new JButton(Application.langue.getProperty("non"));
					JButton annuler = new JButton(Application.langue.getProperty("annuler"));
					final DialogEnregistrement j =new DialogEnregistrement(oui,non,annuler);
					oui.addActionListener(new ActionListener(){
						public void actionPerformed(ActionEvent e)
						{
							j.dispose();
							if(f==null)
							{
								f=getFile(true);
							}
							if(f!=null)
							{
								SauvegardeFichier(f);
								setTitle(NOM_LOGICIEL+ " - "+f.getName());
							}
							importerFichierTexte();
							activationMenus(true);
							dessinOuvert=true;
						}
					});
					non.addActionListener(new ActionListener()
					{
						public void actionPerformed(ActionEvent e)
						{
							j.dispose();
							fermerDessin();
							importerFichierTexte();
							activationMenus(true);
							dessinOuvert=true;
						}
					});
					annuler.addActionListener(new ActionListener(){
						public void actionPerformed(ActionEvent e)
						{
						}
					});
				}
				else
				{
					importerFichierTexte();
				}
			}
		});
		
		exportDot.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent e)
			{
				choixExporterDot();
			}
		});
		
		importDot.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent e)
			{
				if(dessinOuvert)
				{
					JButton oui = new JButton(Application.langue.getProperty("oui"));
					JButton non = new JButton(Application.langue.getProperty("non"));
					JButton annuler = new JButton(Application.langue.getProperty("annuler"));
					final DialogEnregistrement j =new DialogEnregistrement(oui,non,annuler);
					oui.addActionListener(new ActionListener(){
						public void actionPerformed(ActionEvent e)
						{
							j.dispose();
							if(f==null)
							{
								f=getFile(true);
							}
							if(f!=null)
							{
								SauvegardeFichier(f);
								setTitle(NOM_LOGICIEL+ " - "+f.getName());
							
							}
							importerFichierDot();
						}
					});
					non.addActionListener(new ActionListener()
					{
						public void actionPerformed(ActionEvent e)
						{
							j.dispose();
							fermerDessin();
							importerFichierDot();
						}
					});
					annuler.addActionListener(new ActionListener(){
						public void actionPerformed(ActionEvent e)
						{
						}
					});
				}
				else
				{
					importerFichierDot();
				}
			}
		});
		
		//code pour le menu item fermer
		fermer.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent e)
			{
				if(dessinOuvert)
				{
					quitter_enregistrer(false);
				}
				else
					fermerDessin();
				dessinOuvert=false;
			}

		});


		//code pour le menu item quitter
		quitter.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent eve){
				if(dessinOuvert)
					quitter_enregistrer(true);
				else
					fermerApplication(true);
				dessinOuvert=false;
			}
		});

		//code pour le menu item ouvrir
		ouvrir.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent e)
			{
				if(dessinOuvert)
				{
					JButton oui = new JButton(Application.langue.getProperty("oui"));
					JButton non = new JButton(Application.langue.getProperty("non"));
					JButton annuler = new JButton(Application.langue.getProperty("annuler"));
					final DialogEnregistrement j =new DialogEnregistrement(oui,non,annuler);
					oui.addActionListener(new ActionListener(){
						public void actionPerformed(ActionEvent e)
						{
							j.dispose();
							if(f==null)
							{
								f=getFile(true);
							}
							if(f!=null)
							{
								SauvegardeFichier(f);
							}
							f=getFile(false);
							if(f!=null)
							{
								OuvrirFichier(f);
							}
						}
					});
					non.addActionListener(new ActionListener()
					{
						public void actionPerformed(ActionEvent e)
						{
							j.dispose();

							fermerDessin();
							f=getFile(false);
							if(f!=null)
							{
								OuvrirFichier(f);
							}
						}
					});
					annuler.addActionListener(new ActionListener(){
						public void actionPerformed(ActionEvent e)
						{

						}
					});
				}
				else
				{
					f=getFile(false);
					if(f!=null)
					{
						OuvrirFichier(f);
					}
				}
			}
		});

		//code pour le menu item enregistrer
		enregistrer.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent e)
			{
				if(f==null)
				{
					f=getFile(true);
				}
				if(f!=null)
				{
					SauvegardeFichier(f);
				}
			}
		});

		//code pour le menu item enregistrer sous
		enregistrer_sous.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent e) 
			{
				f=getFile(true);
				if(f!=null)
				{
					SauvegardeFichier(f);
				}

			}
		});

		//code pour le menu item d'impression
		imprimer.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e)
			{
				Properties prop = new Properties();
				prop.put("awt.print.paperSize", "a4");
				prop.put("awt.print.destination", "printer");
				PrintJob pjob = getToolkit().getPrintJob(instance,"Printing_Test", prop);
				if(pjob!=null)
				{
					Graphics pg = pjob.getGraphics();
					Component c = et.jsp.getViewport().getView();
					c.printAll(pg);
					pg.dispose();
					pjob.end();
				}
			}
		});
		
		apropos.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent e)
			{
				APropos a = new APropos();
				a.repaint();
			}
		});
		
		
		//on ajoute les menu item au jmenu
		menuFichier.add(nouveau);
		menuFichier.add(ouvrir);
		menuFichier.addSeparator();
		menuFichier.add(enregistrer);
		menuFichier.add(enregistrer_sous);
		menuFichier.addSeparator();
		menuFichier.add(importer);
		menuFichier.add(importeTexte);
		menuFichier.add(importDot);
		menuFichier.add(exporter);
		menuFichier.add(exportDot);
		menuFichier.addSeparator();
		menuFichier.add(imprimer);
		menuFichier.addSeparator();
		menuFichier.add(fermer); 
		menuFichier.add(quitter);
		
		menuCalculs.add(requetes);
		menuCalculs.add(simulation);
		menuCalculs.add(coherenceBornes);
		menuCalculs.add(ensembleSolution);
		menuCalculs.add(courbe);

		requetes.add(arriveeEtat);
		requetes.add(arriveeChemin);
		requetes.add(uneFoisParUnEtat);
		requetes.add(uneFoisParUnChemin);
		requetes.add(auMoinsUneFoisParUnEtat);
		requetes.add(auMoinsUneFoisParUnChemin);
		requetes.add(aucuneFoisParUnEtat);
		requetes.add(aucuneFoisParUnChemin);
		
		menuOptions.add(resFile);
		menuOptions.add(contrainteProba);
		menuOptions.add(fileDirectory);
		menuOptions.add(avance);
		avance.add(paramRl);
		avance.add(paramDef);
		avance.add(sauvParam);
		
		menuAffichage.add(resGraphe);
		menuAffichage.add(organiserGraphe);
		menuAffichage.add(organiserGrappa);
		menuAffichage.add(tailleEtat);
		menuAffichage.add(couleurEtat);
		menuAffichage.add(couleurTransition);

		aide.add(apropos);
		
		final JMenuBar mb = new JMenuBar();
		mb.add(menuFichier);
		mb.add(menuCalculs);
		mb.add(menuOptions);
		mb.add(menuAffichage);
		mb.add(aide);
		setJMenuBar(mb);

		
		courbe.addActionListener(new ActionListener()
		{
			@SuppressWarnings("static-access")
			public void actionPerformed(ActionEvent e)
			{
				StartCalculCourbeGraph tccg =new StartCalculCourbeGraph(et.dessG.graphe,et.dessG.graphe.getlisteCoutMult(),et.dessG.EPSILON);
				ltc.add(tccg);
				et.menuT.majThread();
				tccg.start();
			}
		});
	
		paramRl.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent e)
			{
				new DialogParametreRL();
			}
		});
		
		paramDef.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e)
			{
				RechercheLocale.setParametre(200, 0.001, 100, 500);
				DessinGraphe.setEpsilon(0.0000001);
			}
		});
		
		sauvParam.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e)
			{
				Application.config.setProperty("randomrestart",String.valueOf(RechercheLocale.getRS()));
				Application.config.setProperty("pas",String.valueOf(RechercheLocale.getPas()));
				Application.config.setProperty("nbiteration",String.valueOf(RechercheLocale.getNBI()));
				Application.config.setProperty("tailleliste",String.valueOf(RechercheLocale.getTL()));
				Application.config.setProperty("epsilon", String.valueOf(DessinGraphe.EPSILON));
				try 
				{
					FileOutputStream on = new FileOutputStream("config.ini");
					Application.config.store(on, null);
				}
				catch (FileNotFoundException e1) 
				{
					e1.printStackTrace();
				} catch (IOException er) {
					er.printStackTrace();
				}
			}
		});
		
		
		
		couleurEtat.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent e)
			{
				 Color c = JColorChooser.showDialog(getInstance(), Application.langue.getProperty("couleuretat"),Color.black);
				 if( c != null )
						et.setColorEtat(c);
				 
			}
		});
		
		couleurTransition.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent e)
			{
				 Color c = JColorChooser.showDialog(getInstance(), Application.langue.getProperty("couleurtransition"),Color.black);
				if( c != null )
					et.setColorTransition(c);
						
				 
			}
		});
		
		tailleEtat.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent e)
			{
				final JDialog taille = new JDialog(getInstance());
				taille.setTitle(Application.langue.getProperty("rayonetat"));
				taille.setLocationRelativeTo(getInstance());
				JPanel jp = new JPanel();
				final JSlider js = new JSlider(20,40);
				js.setPaintTicks(true);//dessine les traits de l'echelle
				js.setMinorTickSpacing(1);//espace minimum entre les petits traits des graduations
				js.setMajorTickSpacing(10);//espace maximum entre les grands traits des graduations
				final JIntField jf = new JIntField(6);
				jf.setInt(js.getValue());
				jp.add(js);
				jp.add(jf);
				taille.add(jp);
				js.addChangeListener(new ChangeListener(){
					public void stateChanged(ChangeEvent e)
						{
						jf.setInt(js.getValue());
						et.setRayon(js.getValue());
						}
				});
				taille.setSize(250,100);
				taille.setVisible(true);
			}
		} );
		
		resGraphe.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent e)
			{
				et.setResultatGraphe(resGraphe.getState());
				repaint();
			}
		});

		organiserGraphe.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent e)
			{
				et.dessG.graphe.definePositionState();
				et.dessG.repaint();
			}
		});
		
		organiserGrappa.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent e)
			{
				et.reorganiserGrappa();
				repaint();
			}
		});
		
		contrainteProba.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent e)
			{
				new DialogListeRelationProba(et.dessG.graphe);
			}
		});
		
		simulation.addActionListener(new ActionListener()
		{
			
			public void actionPerformed(ActionEvent e)
			{
				new FenetreSimulation(et.dessG);
			}
		});
		
		coherenceBornes.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent e)
			{
				StartCoherenceBornes scb = new StartCoherenceBornes(et.dessG);
				ltc.add(scb);
				et.menuT.majThread();
				scb.start();
			}
		});
		
		ensembleSolution.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent e)
			{
				//new StartEnsembleSolution(et.dessG.graphe,et.dessG.methodes).start();
				final JDialog jd = new JDialog(getInstance(),Application.langue.getProperty("volume"));
				final JIntField jif = new JIntField(10);
				JLabel jl = new JLabel(Application.langue.getProperty("nbpointsvolume"));
				jif.setInt(15);
				JButton launch = new JButton(Application.langue.getProperty("lancer"));
				launch.addActionListener(new ActionListener(){
					public void actionPerformed(ActionEvent e)
					{
						int nbp = jif.getInt();
						Matrice<Intervalle> mi = et.dessG.graphe.construireMatriceIntervalle();
						int dim = Matrice.Dimension(mi);
						if(nbp>dim)
						{
							StartCalculVolume cv = new StartCalculVolume(mi,et.dessG.graphe.getNombreEtats(),et.dessG.methodes,nbp,dim,false);
							ltc.add(cv);
							et.menuT.majThread();
							cv.start();
							repaint();
							jd.dispose();
						}
						else
							new ErreurDialog(Application.langue.getProperty("erreurvolume"));
					}
				});
				JButton cancel = new JButton(Application.langue.getProperty("annuler"));
				cancel.addActionListener(new ActionListener()
				{
					public void actionPerformed(ActionEvent e)
					{
						jd.dispose();
					}
				});
				JPanel jp = new JPanel();
				JPanel pBoutons = new JPanel();
				jp.add(jl);
				jp.add(jif);
				pBoutons.add(launch);
				pBoutons.add(cancel);
				jd.add(jp,BorderLayout.CENTER);
				jd.add(pBoutons,BorderLayout.SOUTH);
				jd.setLocationRelativeTo(getInstance());
				jd.setSize(300,125);
				jd.setVisible(true);
			}
		});

		aucuneFoisParUnEtat.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e)
			{
				if(et.dessG.graphe.getNombreEtats()==0)
					Erreur(Application.langue.getProperty("erreurpasdetat"));
				else
					new DialogProbaNePasPasser(et.dessG,false);
			}
		});
		
		aucuneFoisParUnChemin.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e)
			{
				if(et.dessG.getChemins().size()==0)
					Erreur(Application.langue.getProperty("erreurpasdechemin"));
				else
					new DialogProbaNePasPasser(et.dessG,true);
			}
		});
		
		uneFoisParUnEtat.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent e)
			{
				if(et.dessG.graphe.getNombreEtats()==0)
					Erreur(Application.langue.getProperty("erreurpasdetat"));
				else
					new DialogUneFoisParUnEtat(et.dessG,true);
			}
		});
		
		uneFoisParUnChemin.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent e)
			{
				if(et.dessG.getChemins().size()==0)
					Erreur(Application.langue.getProperty("erreurpasdechemin"));
				else
					new DialogUneFoisParUnChemin(et.dessG,true);
			}
		});
		
		auMoinsUneFoisParUnEtat.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent e)
			{
				if(et.dessG.graphe.getNombreEtats()==0)
					Erreur(Application.langue.getProperty("erreurpasdetat"));
				else
					new DialogUneFoisParUnEtat(et.dessG,false);
			}
		});
		
		
		auMoinsUneFoisParUnChemin.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent e)
			{
				if(et.dessG.getChemins().size()==0)
					Erreur(Application.langue.getProperty("erreurpasdechemin"));
				else
					new DialogUneFoisParUnChemin(et.dessG,false);
			}
		});
		
		resFile.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e)
			{
				et.setResultatFile(resFile.getState());
				if(pathFileResult==null)
					pathFileResult = "result.csv";
				try
				{
					BufferedWriter fic = new BufferedWriter(new FileWriter(pathFileResult));
					fic.write(Application.langue.getProperty("mess1csv"));
					fic.newLine();
					fic.write("-----------------------------");
					fic.newLine();
					fic.write(et.dessG.graphe.construireMatriceIntervalle().toCSV());
					fic.newLine();
					fic.newLine();
					fic.write(Application.langue.getProperty("mess2csv"));
					fic.newLine();
					fic.write("-----------------------------------------");
					fic.newLine();
					fic.close();
				}
				catch(Exception ex)
				{
					ex.printStackTrace();
				}
			}
		});

		fileDirectory.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e)
			{
				pathFileResult = getFileResult();
				if(pathFileResult!=null)
				{
					et.dessG.setPathFile(pathFileResult);
					resFile.setEnabled(true);
				}
			}
		});
		
		arriveeEtat.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent e)
			{
				if(et.dessG.graphe.getNombreEtats()==0)
					Erreur(Application.langue.getProperty("erreurpasdetat"));
				else
					new DialogArriveeEtat(et.dessG,false);
			}
		});
		
		arriveeChemin.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e)
			{
				if(et.dessG.getChemins().size()==0)
					Erreur(Application.langue.getProperty("erreurpasdechemin"));
				else
					new DialogArriveeEtat(et.dessG,true);
			}
		});


		//code pour l'item nouveau
		nouveau.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent e)
			{
				//on demande si on doit fermer le dessin ou non
				if(dessinOuvert)
				{
					JButton oui = new JButton(Application.langue.getProperty("oui"));
					JButton non = new JButton(Application.langue.getProperty("non"));
					JButton annuler = new JButton(Application.langue.getProperty("annuler"));
					final DialogEnregistrement j = new DialogEnregistrement(oui,non,annuler);
					oui.addActionListener(new ActionListener(){
						public void actionPerformed(ActionEvent e)
						{j.dispose();
						if(f==null)
						{
							f=getFile(true);
						}
						if(f!=null)
						{
							SauvegardeFichier(f);
						}
						DessinGraphe dg=new DessinGraphe();
						et.setDessG(dg);
						dg.repaint();
						setVisible(true);
						setTitle(NOM_LOGICIEL);
						dessinOuvert=true;
						activationMenus(true);
						}
					});
					non.addActionListener(new ActionListener()
					{
						public void actionPerformed(ActionEvent e)
						{
							j.dispose();
							fermerDessin();
							DessinGraphe dg=new DessinGraphe();
							et.setDessG(dg);
							dg.repaint();
							setVisible(true);
							setTitle(NOM_LOGICIEL);

							dessinOuvert=true;
							activationMenus(true);
						}
					});
					annuler.addActionListener(new ActionListener(){
						public void actionPerformed(ActionEvent e)
						{

						}
					});
				}
				else
				{
					DessinGraphe dg=new DessinGraphe();
					et.setDessG(dg);
					dg.repaint();
					setVisible(true);

					dessinOuvert=true;
					activationMenus(true);
				}
			}
		});
	}

	public void activationMenus(boolean b)
	{
		enregistrer.setEnabled(b);
		enregistrer_sous.setEnabled(b);
		menuCalculs.setEnabled(b);
		menuAffichage.setEnabled(b);
		menuOptions.setEnabled(b);
		exporter.setEnabled(b);
		exportDot.setEnabled(b);
		fermer.setEnabled(b);
		imprimer.setEnabled(b);
		if(b && resGraphe.isSelected())
			resGraphe.setSelected(false);
	}

	
	/**
	 * Methode pour importer un fichier xml qui utilise la norme sbml
	 * On recupère le fichier à l'aide d'un JFileChooser
	 */
	public void Importer()
	{
		JFileChooser fc = new JFileChooser();
		MonFiltre mf = new MonFiltre("xml");
		fc.setFileFilter(mf);
		fc.setApproveButtonText(Application.langue.getProperty("import"));
		fc.addChoosableFileFilter(mf);
		int	returnval = fc.showDialog(FenetrePrincipale.this,Application.langue.getProperty("import"));
		if(returnval==JFileChooser.APPROVE_OPTION)
		{
			f=fc.getSelectedFile();
			String s=f.getName();
			String suf=new String();
			int i = s.lastIndexOf('.');

			if (i > 0 &&  i < s.length() - 1) {

				suf = s.substring(i+1).toLowerCase();
				if(suf.compareTo("xml")!=0)
					Erreur(Application.langue.getProperty("erreursbml"));
				else 
					OuvrirSBMLFile(f);
			}
		}
	}
	
	/**
	 * Methode qui permet d'importer un fichier texte
	 * correspondant  un graphe
	 */
	public void importerFichierTexte()
	{
		if(et==null)
		{
			et = new EspaceTravail();
		}
		DessinGraphe dessG = new DessinGraphe();
		ChargementFichierGraphe cf = new ChargementFichierGraphe(instance);
		Graphe g = cf.chargeGraphe();
		if(g!=null)
		{
			g.reorganiserGrappa();
			dessG.setGraphe(g);
			et.setDessG(dessG);
			dessG.repaint();
			activationMenus(true);
			setVisible(true);
			dessinOuvert=true;
		}
	}
	
	/**
	 * Methode pour importer un fichier DOT
	 */
	public void importerFichierDot()
	{
		if(et==null)
		{
			et = new EspaceTravail();
		}
		DessinGraphe dessG = new DessinGraphe();
		ImportationDot id = new ImportationDot(instance);
		Graphe g = id.chargeGraphe(); 
		if(g!=null)
		{
			dessG.setGraphe(g);
			et.setDessG(dessG);
			dessG.repaint();
			activationMenus(true);
			setVisible(true);
			dessinOuvert=true;
		}
	}
	
	/**
	 * Methode qui permet d'exporter le graphe au format .dot
	 */
	public void choixExporterDot()
	{
		JFileChooser fc = new JFileChooser();
		MonFiltre mf = new MonFiltre("dot");
		fc.setFileFilter(mf);
		fc.setApproveButtonText(Application.langue.getProperty("exporter"));
		fc.addChoosableFileFilter(mf);
		int returnval = fc.showDialog(FenetrePrincipale.this,Application.langue.getProperty("export"));
		if(returnval==JFileChooser.APPROVE_OPTION)
		{
			f=fc.getSelectedFile();
			f=fc.getSelectedFile();
			String s=f.getName();
			String suf=new String();
			int i = s.lastIndexOf('.');

			if (i > 0 &&  i < s.length() - 1) {

				suf = s.substring(i+1).toLowerCase();
				if(suf.compareTo("dot")!=0)
				{
					FenetrePrincipale.this.Erreur(Application.langue.getProperty("erreurdot"));
				}
				else 
					exporterDot(f);
			}
			else
			{
				f=new File(f.getPath()+".dot");
				exporterDot(f);
			}
		}
	}
	public void exporterDot(File f)
	{
		
		try 
		{
			FileWriter fw = new FileWriter(f);
			fw.write(et.dessG.graphe.toDot());
			fw.close();
		} 
		catch (IOException e) 
		{
			e.printStackTrace();
		}
		
	}
	
	/**
	 * Methode d'exporter le graphe sous format xml en respectant la norme sbml
	 * On récupère le fichier à l'aide d'un JFileChooser
	 */
	public void Exporter()
	{
		JFileChooser fc = new JFileChooser();
		MonFiltre mf = new MonFiltre("xml");
		fc.setFileFilter(mf);
		fc.setApproveButtonText(Application.langue.getProperty("exporter"));
		fc.addChoosableFileFilter(mf);
		int returnval = fc.showDialog(FenetrePrincipale.this,Application.langue.getProperty("exporter"));
		if(returnval==JFileChooser.APPROVE_OPTION)
		{
			f=fc.getSelectedFile();
			f=fc.getSelectedFile();
			String s=f.getName();
			String suf=new String();
			int i = s.lastIndexOf('.');

			if (i > 0 &&  i < s.length() - 1) {

				suf = s.substring(i+1).toLowerCase();
				if(suf.compareTo("xml")!=0)
				{
					FenetrePrincipale.this.Erreur(Application.langue.getProperty("erreursbml"));
				}
				else 
					SauvegarderSBMLFile(f);
			}
			else
			{
				f=new File(f.getPath()+".xml");
				SauvegarderSBMLFile(f);
			}
		}

	}

	/**
	 * Methode permettant de lire un fichier xml respectant la norme sbml
	 * @param f
	 */
	public void OuvrirSBMLFile(File f)
	{
		setTitle(NOM_LOGICIEL+ " - "+f.getName());
		if(et==null)
		{
			et = new EspaceTravail();
		}
		DessinGraphe dessG = new DessinGraphe();
		if (dessG.ChargerSBMLFile(f))
		{
			et.setDessG(dessG);
			activationMenus(true);
			dessinOuvert=true;
			dessG.repaint();
			setVisible(true);
		}
	}

	/**
	 * Methode permettant d'afficher une Erreur dans une ErreurDialog
	 * @param s
	 */
	public void Erreur(String s)
	{
		JOptionPane.showMessageDialog(this, s, Application.langue.getProperty("erreur"), JOptionPane.ERROR_MESSAGE);
	}

	/**
	 * Methode permettant de sauvegarder le graphe dans un ficher xml respectant
	 *  la norme sbml
	 * @param f
	 */
	public void SauvegarderSBMLFile(File f)
	{
		
		SBMLWriter sw= new SBMLWriter();
		SBMLReader sr = new SBMLReader();
		SBMLDocument d = sr.readSBML(f.getPath());
		Model m = new Model("m1");
		Compartment c = new Compartment("c1");
		m.addCompartment(c);
		et.SauvegarderSBMLFile(m);
		d.setModel(m);
		String st=sw.writeToString(d);
		FileWriter fw;
		try {
			fw = new FileWriter(f);
			fw.write(st);
			fw.flush();
			fw.close();
		} catch (IOException e) {
			e.printStackTrace();
		}

	}

	/**
	 * Loads the SWIG generated libsbml Java module when this class is
	 * loaded.
	 */
	static
	{
		System.loadLibrary("sbmlj");
	}

	/**
	 * Methode permettant d'ouvrir un fichier de type .gda
	 * @param f
	 */
	public void OuvrirFichier(File f)
	{
		FileInputStream fis;
		try {
			fis = new FileInputStream(f);
			setTitle(NOM_LOGICIEL+ " - "+f.getName());
			ObjectInputStream ois = new ObjectInputStream(fis);
			DessinGraphe dg = (DessinGraphe)ois.readObject();
			dg.ouverture_fichier();
			et.setDessG(dg);
			dg.repaint();
			setVisible(true);
			activationMenus(true);
			dessinOuvert=true;

		} catch (FileNotFoundException e1) {
			e1.printStackTrace();
		} catch (IOException e1) {
			((Throwable) e1).printStackTrace();
		} catch (ClassNotFoundException e1) {
			e1.printStackTrace();
		} catch(InstantiationError e){
			JOptionPane.showMessageDialog(instance,Application.langue.getProperty("erreurgda"),Application.langue.getProperty("erreur"), JOptionPane.ERROR_MESSAGE);
			dessinOuvert = false;
		}
	}

	/**
	 *  Methode permettant de sauvegarder un fichier sous format gda
	 * @param f
	 */
	public void SauvegardeFichier(File f)
	{
		FileOutputStream fos;
		try {
			fos = new FileOutputStream(f);

			setTitle(NOM_LOGICIEL+ " - "+f.getName());

			ObjectOutputStream oos = new ObjectOutputStream(fos);

			oos.writeObject(et.dessG);
		} catch (FileNotFoundException e1) {
			e1.printStackTrace();
		} catch (IOException e1) {
			((Throwable) e1).printStackTrace();
		}
	}

	/**
	 *  Methode permettant de récupérer un fichier de type gda
	 * Le boolean x est à true si on veut enregistrer et à faux sinon
	 * @param x
	 * @return File
	 */	public File getFile(boolean x)
	 {
		 JFileChooser fc = new JFileChooser();
		 MonFiltre mf = new MonFiltre("gda");
		 fc.setFileFilter(mf);
		 fc.addChoosableFileFilter(mf);
		 int returnval;
		 if(x)
		 {
			 returnval = fc.showSaveDialog(FenetrePrincipale.getInstance());
		 }
		 else 
		 {
			 returnval = fc.showOpenDialog(FenetrePrincipale.getInstance());
		 }
		 if(returnval == JFileChooser.APPROVE_OPTION)
		 {
			 f=fc.getSelectedFile();
			 if(x)
			 {
				 String s=f.getName();
				 String suf=new String();
				 int i = s.lastIndexOf('.');

				 if (i > 0 &&  i < s.length() - 1) {

					 suf = s.substring(i+1).toLowerCase();
					 if(suf.compareTo("gda")!=0)
					 {
						 JDialog j = new JDialog();
						 JLabel jl = new JLabel(Application.langue.getProperty("erreurgdafile"));
						 j.add(jl);
						 j.setSize(400,100);
						 j.setVisible(true);
						 j.setLocationRelativeTo(FenetrePrincipale.getInstance());
						 return null;
					 }
					 else return f;
				 }
				 else
				 {
					 f=new File(f.getPath()+".gda");
					 return f;
				 }

			 }
			 else
			 {
				 String s=f.getName();
				 String suf=new String();
				 int i = s.lastIndexOf('.');

				 if (i > 0 &&  i < s.length() - 1) 
				 {

					 suf = s.substring(i+1).toLowerCase();
					 if(suf.compareTo("gda")!=0)
					 {
						 Erreur(Application.langue.getProperty("erreurgdafile"));
						 return null;
					 }
					 else return f;
				 }
			 }
		 }
		 return null;
	 }

	 /**
	  * Methode pour recuperer l'emplacement du fichier resultat de la recherche locale
	  */ 
	 public String getFileResult()
	 {
		 JFileChooser fc = new JFileChooser();
		 fc.showSaveDialog(this);
		 File f;
		 f=fc.getSelectedFile();
		 if(f != null)
		 {
			 String s=f.getPath();
			 String suf=new String();
			 int i = s.lastIndexOf('.');
			 if (i > 0 &&  i < s.length() - 1) 
			 {
				 suf = s.substring(i+1).toLowerCase();
				 if(suf.compareTo("csv")!=0)
				 {
					 JOptionPane.showMessageDialog(getParent(), Application.langue.getProperty("erreurcsv"), Application.langue.getProperty("erreur"), JOptionPane.ERROR_MESSAGE);
					 return null;
				 }
				 else return f.getPath();
			 }
			 else
			 {
				 return f.getPath()+".csv";
			 }
		 }
		 return null;
	 }


	 /**
	  * Methode permettant de fermer la fenêtre
	  */
	 public void fermerApplication(boolean confirm)
	 {
		 if(confirm)
		 {
			 JButton oui = new JButton(Application.langue.getProperty("oui"));
			 JButton non = new JButton(Application.langue.getProperty("non"));
			 oui.addActionListener(new ActionListener(){
				 public void actionPerformed(ActionEvent e)
				 {
					 dessinOuvert = false;
					 dispose();
					 System.exit(0);
				 }
			 });
			 new DialogConfirmeQuitter(oui, non);
		 }
		 else
		 {
			 dispose();
			 System.exit(0);
		 }
	 }

}
