/*
Copyright Bourdon Jeremie 
contributors : Giraudet Etienne, Docet Eric

jeremie.bourdon@univ-nantes.fr
eric.docet@etu.univ-nantes.fr
etienne.giraudet@etu.univ-nants.fr

This software is a computer program whose purpose is to do computations
on genetic graphs.

This software is governed by the CeCILL license under French law and
abiding by the rules of distribution of free software.  You can use, 
modify and/ or redistribute the software under the terms of the CeCILL
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL license and that you accept its terms.
*/

package ihm.dessin;

import ihm.Application;
import ihm.intervalle.Intervalle;
import ihm.intervalle.IntervalleGraphique;
import ihm.transition.Transition;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.util.ArrayList;

import javax.swing.JButton;

import dialog.DialogIntervalle;
import dialog.ErreurDialog;
/**
 * Classe qui herite de la classe DessinGrapheState
 * et qui correspond a l'etat ou on ajoute un intervalle
 * @author Docet Giraudet
 *
 */
public class DessinGrapheStateAddIntervalle extends DessinGrapheState{
	
	public DessinGrapheStateAddIntervalle(DessinGraphe dg)
	{
		super(dg);
	}
	/**
	 * On redefini la methode en rajouter un intervalle a la transition
	 */
	public  void mouseClicked(MouseEvent e) {
		//EtatGraphique etat=_dg.rechercheEtatGraphique(e.getX(),e.getY());
		final Transition transition = _dg.rechercheTransitionGraphique(e.getX(),e.getY());
		 if (transition!=null)
		{//l'utilisateur a clique sur une transition
			final Intervalle i = new Intervalle("");
			final IntervalleGraphique ig = new IntervalleGraphique(i);
			JButton bValider = new JButton(Application.langue.getProperty("valider"));
			new DialogIntervalle(ig,bValider,Application.langue.getProperty("it")+(transition.getIntervalles().size()+1), transition);
			bValider.addActionListener(new ActionListener(){
				public void actionPerformed(ActionEvent e)
				{
					if(contrainteNomIntervalleDifferent(ig.getDescription(), transition))
						transition.addIntervalle(i);
					else
						new ErreurDialog(Application.langue.getProperty("erreurDGSAI1"));
				}
			});
			
		}
		//on retourne a l'etat de defaut
		_dg.setState(new DessinGrapheStateDefault(_dg));
	}
	
	public  void mousePressed(MouseEvent e) {
	}
	
	public  void mouseReleased(MouseEvent e) {
	}
	
	public  void mouseEntered(MouseEvent arg0) {
	}

	public  void mouseExited(MouseEvent e) {
	}
	
	public  void mouseDragged(MouseEvent e) {
	}
	
	public  void mouseMoved(MouseEvent arg0) {
	}
	
	public static boolean contrainteNomIntervalleDifferent(String nom,Transition f)
	{
		ArrayList<Intervalle> intervalles=f.getIntervalles();
		for(int i=0;i<intervalles.size();i++)
		{
			if(intervalles.get(i).getDescription() == nom)
				return false;
		}
		return true;
		
	}
}
