/*
Copyright Bourdon Jeremie 
contributors : Giraudet Etienne, Docet Eric

jeremie.bourdon@univ-nantes.fr
eric.docet@etu.univ-nantes.fr
etienne.giraudet@etu.univ-nants.fr

This software is a computer program whose purpose is to do computations
on genetic graphs.

This software is governed by the CeCILL license under French law and
abiding by the rules of distribution of free software.  You can use, 
modify and/ or redistribute the software under the terms of the CeCILL
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL license and that you accept its terms.
*/

package ihm.dessin;

import ihm.Application;
import ihm.FenetrePrincipale;
import ihm.etat.EtatGraphique;
import ihm.transition.Transition;
import java.awt.event.MouseEvent;

import javax.swing.JOptionPane;

import dialog.DialogAjoutTransition;
/**
 * Classe qui herite de la classe DessinGrapheState et qui correspond
 * a l'etat ou on ajoute une transition
 * @author Docet Giraudet
 *
 */
public class DessinGrapheStateAddTransition extends DessinGrapheState{

	private EtatGraphique etat;

	public DessinGrapheStateAddTransition(DessinGraphe dg)
	{
		super(dg);
	}

	public  void mouseClicked(MouseEvent e) {
	}
	/**
	 * on redefinie la methode dans laquelle on cree une fleche si
	 * l'utilisateur a clique sur un etat
	 */
	public  void mousePressed(MouseEvent e) {
		Transition f;
		etat=_dg.rechercheEtatGraphique(e.getX(),e.getY());
		f=new Transition();
		f.setDepart(e.getX(),e.getY());
		if(etat!=null)
		{
			etat.addTransitionsSortantes(f);
			_dg.setEtatSelectionne(etat);
			_dg.getListeTransition().add(f);
			f.setArrivee(e.getX(),e.getY());
			f.setEtatDepart(etat);
			_dg.setLayout(null);
			_dg.add(f);
			_dg.repaint();
		}
	}

	/**
	 * On redefini cette methode ou on supprime la fleche
	 * s'il l'utilisateur a lache le bouton de la souris
	 * en dehors d'un etat sinon on recupere l'etat d'arrivee
	 * auquel on ajoute la fleche
	 */
	public  void mouseReleased(MouseEvent e) {
		if(_dg.etat_selectionne!=null)
		{
			EtatGraphique etat = _dg.rechercheEtatGraphique(e.getX(),e.getY());
			final Transition f = _dg.getListeTransition().get(_dg.getListeTransition().size()-1);
			if(etat==null)
			{
				_dg.getEtatSelectionne().removeTransitionsSortantes(f);
				_dg.getListeTransition().remove(f);
			}
			else
			{
				if(_dg.graphe.rechercheTransition(_dg.getListeTransition().get(_dg.getListeTransition().size()-1).getEtatDepart(), etat) == null)
				{
					etat.addTransitionsEntrantes(f);
					f.setEtatArrivee(etat);
					f.disposeTransition();
					//Ajout de l'intervalle par dfault
					new DialogAjoutTransition(f);
				}
				else
				{
					_dg.getEtatSelectionne().removeTransitionsSortantes(f);
					_dg.getListeTransition().remove(f);
					_dg.repaint();
					JOptionPane.showMessageDialog(FenetrePrincipale.getInstance(), Application.langue.getProperty("erreurDGSAT1"), Application.langue.getProperty("erreur"), JOptionPane.ERROR_MESSAGE);
				}
			}
		}
		_dg.repaint();
		_dg.setEtatSelectionne(null);
		_dg.setState(new DessinGrapheStateDefault(_dg));

	}

	public  void mouseEntered(MouseEvent arg0) {
	}

	public  void mouseExited(MouseEvent e) {
	}
	/**
	 * On redefini la methode dans laquelle on dessine la fleche au fur et a 
	 * mesure que l'utilisateur bouge la souris
	 */
	public  void mouseDragged(MouseEvent e) {
		if(etat!=null)
		{
			_dg.getListeTransition().get(_dg.getListeTransition().size()-1).setArrivee(e.getX(),e.getY());
			_dg.repaint();
		}
	}

	public  void mouseMoved(MouseEvent arg0) {
	}
}
