/*
Copyright Bourdon Jeremie 
contributors : Giraudet Etienne, Docet Eric

jeremie.bourdon@univ-nantes.fr
eric.docet@etu.univ-nantes.fr
etienne.giraudet@etu.univ-nants.fr

This software is a computer program whose purpose is to do computations
on genetic graphs.

This software is governed by the CeCILL license under French law and
abiding by the rules of distribution of free software.  You can use, 
modify and/ or redistribute the software under the terms of the CeCILL
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL license and that you accept its terms.
*/

package ihm.dessin;

import ihm.etat.EtatGraphique;
import java.awt.Dimension;
import java.awt.event.MouseEvent;
/**
 * Classe qui herite de la classe DessinGrapheState et qui correspond
 * a l etat ou aucun bouton n'a ete selectionne
 * @author Docet Giraudet
 *
 */
public class DessinGrapheStateDefault extends DessinGrapheState{

	public DessinGrapheStateDefault(DessinGraphe dg)
	{
		super(dg);
	}
	
	public void mouseClicked(MouseEvent e) {
	}
	/**
	 * On redefini la methode ou l'on selectionne l'objet ou l'utilisateur a clique
	 */
	public  void mousePressed(MouseEvent e) {
		EtatGraphique etat=_dg.rechercheEtatGraphique(e.getX(),e.getY());
		if(etat!=null)
			{
				_dg.setEtatSelectionne(etat);
			}
		if(_dg.selectPoint(e.getX(),e.getY()))
		{
			_dg.setPointSelectionne(true);
			_dg.repaint();
		}
	}
	/**
	 * on redefinie la methode ou on deselectionne les objets
	 */
	public  void mouseReleased(MouseEvent e) {
		_dg.setEtatSelectionne(null);
		_dg.setPointSelectionne(false);
	}
	
	public  void mouseEntered(MouseEvent arg0) {
	}

	public  void mouseExited(MouseEvent e) {
	}
	/**
	 * On redefinie la methode ou si un etat est selectionne alors on 
	 * fait bouger l'etat en suivant la souris a condition que la souris ne sorte
	 * pas du rectangle de dessin
	 * ou si on a selectionne le coin inferieur droit du rectangle dans ce
	 * cas on agrandi le rectangle et la taille du dessingraphe pour que le scroll
	 * s'agrandisse
	 */
	public  void mouseDragged(MouseEvent e) {
		RectangleDessin r = _dg.getRect();
		if((e.getX()<r.getLarg())&&(e.getY()<r.getHaut())&&(e.getX()>0)&&(e.getY()>0))
		{
			if(_dg.getEtatSelectionne()!=null)
			{
				EtatGraphique et = _dg.getEtatSelectionne();
				et.setX(e.getX()-et.getRayon());
				et.setY(e.getY()-et.getRayon());
				_dg.repaint();
			}
			
		}
		if(_dg.getPointSelectionne())
		{
			r.setLarg(e.getX());
			r.setHaut(e.getY());
			_dg.setSize(new Dimension(r.getLarg(),r.getHaut()));
			_dg.repaint();
		}
	}
	
	public  void mouseMoved(MouseEvent arg0) {
	}
	
}
