/*
Copyright Bourdon Jeremie 
contributors : Giraudet Etienne, Docet Eric

jeremie.bourdon@univ-nantes.fr
eric.docet@etu.univ-nantes.fr
etienne.giraudet@etu.univ-nants.fr

This software is a computer program whose purpose is to do computations
on genetic graphs.

This software is governed by the CeCILL license under French law and
abiding by the rules of distribution of free software.  You can use, 
modify and/ or redistribute the software under the terms of the CeCILL
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL license and that you accept its terms.
*/

package ihm.dessin;

import java.awt.Color;
import java.awt.Graphics;
import java.io.Serializable;

import javax.swing.JPanel;
/**
 * Classe qui herite de la classe JPanel et implemente l'interface Serializable
 * qui correspond au à la surface de dessin
 * @author Docet Giraudet
 *
 */
public class RectangleDessin extends JPanel implements Serializable
{

	
	private static final long serialVersionUID = 1L;
	private int _larg;
	private int _haut;
	private int _x;
	private int _y;
	private int _minl;
	private int _minh;
	
	/**
	 * Constructeur du rectangle
	 * @param x abscisse du coin en haut à gauche
	 * @param y ordonne du coin en haut à gauche
	 * @param widht largeur
	 * @param height hauteur
	 */
	public RectangleDessin(int x,int y,int widht,int height)
	{
		_larg=widht;
		_haut=height;
		_x=x;
		_y=y;
	}
	
	/**
	 * Methode pour indiquer la largeur minimum du rectangle
	 * @param x
	 */
	public void setMinimumLarg(int x)
	{
		_minl = x;
	}
	/**
	 * Methode pour indiquer la hauteur minimum du rectangle
	 * @param x
	 */
	public void setMinimumHaut(int x)
	{
		_minh = x;
	}

	/**
	 * Methode pour recuperer l'abscisse du rectangle
	 */
	public int getX()
	{
		return _x;
	}
/**
 * Methode pour recuperer l'ordonnee du rectangle
 */
	public int getY()
	{
		return _y;
	}

	/**
	 * Methode pour recuperer la largeur du rectangle
	 * @return _larg int
	 */
	public int getLarg()
	{
		return _larg;
		
	}

	/**
	 * Methode pour recuperer la hauteur du rectangle
	 * @return _haut int
	 */
	public int getHaut()
	{
		return _haut;
	}
	
	/**
	 * Methode pour modifier la largeur du rectangle
	 * @param x
	 */
	public void setLarg(int x)
	{
		if(x>_minl)
			_larg=x;
	}
	/**
	 * Methode pour modifier la hauteur du rectangle
	 * @param x
	 */
	public void setHaut(int x)
	{
		if(x>_minh)
			_haut=x;
	}

	/**
	 * Methode pour dessiner le rectangle
	 * @param g
	 */
	public void DessineRectangle(Graphics g)
	{
		g.setColor(Color.WHITE);
		g.fillRect(_x,_y,_larg,_haut);
		g.setColor(Color.BLACK);
		g.fillRect(_larg-5,_haut-5,5,5);
	}



	


}
