/*
Copyright Bourdon Jeremie 
contributors : Giraudet Etienne, Docet Eric

jeremie.bourdon@univ-nantes.fr
eric.docet@etu.univ-nantes.fr
etienne.giraudet@etu.univ-nants.fr

This software is a computer program whose purpose is to do computations
on genetic graphs.

This software is governed by the CeCILL license under French law and
abiding by the rules of distribution of free software.  You can use, 
modify and/ or redistribute the software under the terms of the CeCILL
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL license and that you accept its terms.
*/

package ihm.transition;

import java.awt.Graphics;
import java.awt.Point;
/**
 * Classe servant a representer le dessin des boucles sur le graphe
 * @author giraudet-docet
 *
 */
public class Boucle 
{
	private Point centreEtat;
	private Point centreBoucle;
	private int rayonEtat;
	static double pi = Math.PI;
	static final double ECART_ANGLE_BOUCLE = pi / 4;
	
	public Boucle(Point e, Point b, int r)
	{
		centreEtat = e;
		centreBoucle = b;
		rayonEtat = r;
	}
	
	public void dessineBoucle(Graphics g,Double res)
	{
		int x = centreBoucle.x;
		int y = centreBoucle.y;
		int xc = centreEtat.x;
		int yc = centreEtat.y;
		
		double phi = ((pi/2)-ECART_ANGLE_BOUCLE)/2 ;
		int xm1 =  (int) (xc - rayonEtat * Math.cos( phi + ECART_ANGLE_BOUCLE ));
		int ym1 = (int) (yc - rayonEtat * Math.sin( phi + ECART_ANGLE_BOUCLE ));
		Point M1 = new Point(xm1,ym1);
		
		double a = ((ym1-yc)/(xm1-xc));
		double b = ym1 - a * xm1;
		
		int xt = xm1 - 10;
		int yt = (int) (a * xt + b);
		
		int rayon = (int) M1.distance(x,y);
		
		g.drawArc(x-rayon, y-rayon, rayon*2, rayon*2, (int) radToDeg((-phi*rayon)/rayonEtat), (int) (360 - radToDeg((ECART_ANGLE_BOUCLE*rayonEtat)/rayon)));
		
		dessineBoutTransition(g, xt, yt, xm1, ym1);
		if(res!=null)
		{
		String s = res.toString();
		if(s.length()>6)
			s=s.substring(0,6);
		g.drawString(s,x-(g.getFont().getSize()*s.length())/4,y);
		}
	}
	
	
	
	private void dessineBoutTransition(Graphics g, int x1, int y1, int x2, int y2)
	{
		if(!(x1==x2 && y1==y2))
		{
			double x3, y3, x4, y4;
			int l = 10;
			double a = pi/6 ;
			double L = Math.sqrt(Math.pow(x2 - x1, 2) + Math.pow(y2 - y1, 2));
			x3 = x2 - l * ((x2 - x1) / L * Math.cos(a) + (y2 - y1) / L * Math.sin(a));
			y3 = y2 + l * ((x2 - x1) / L * Math.sin(a) - (y2 - y1) / L * Math.cos(a));
			x4 = x2 - l * ((x2 - x1) / L * Math.cos(-a) + (y2 - y1) / L * Math.sin(-a));
			y4 = y2 + l * ((x2 - x1) / L * Math.sin(-a) - (y2 - y1) / L * Math.cos(-a));
			
			g.drawLine(x2, y2, (int) x3, (int) y3);
			g.drawLine(x2, y2, (int) x4, (int) y4);		
		}	
	}
	

	
	public static double radToDeg(double r)
	{
		return (180 * r) / pi;
	}
}
