/*
Copyright Bourdon Jeremie 
contributors : Giraudet Etienne, Docet Eric

jeremie.bourdon@univ-nantes.fr
eric.docet@etu.univ-nantes.fr
etienne.giraudet@etu.univ-nants.fr

This software is a computer program whose purpose is to do computations
on genetic graphs.

This software is governed by the CeCILL license under French law and
abiding by the rules of distribution of free software.  You can use, 
modify and/ or redistribute the software under the terms of the CeCILL
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL license and that you accept its terms.
*/

package thread;

import java.util.ArrayList;

import graphe.Graphe;
import ihm.Application;
import ihm.FenetrePrincipale;
import ihm.dessin.DessinGraphe;

import javax.swing.JButton;
import javax.swing.JOptionPane;

import calcul.Matrice;
import calcul.MethodeResolution;
import calcul.RechercheLocale;
import dialog.DialogAfficheMatrice;
/**
 * Classe representant le calcul d'une solution
 * @author giraudet
 *
 */
public class StartRechercheLocale extends ThreadCalcul{
	
	private Graphe graphe;
	private RechercheLocale rd;
	private JButton jb;
	private static int num=0;
	private int numero;
	private Matrice<Double> m;

	@SuppressWarnings("static-access")
	public StartRechercheLocale(Graphe g,ArrayList<MethodeResolution> mr,double epsilon)
	{
		super();
		graphe=g;
		num++;
		numero=num;
		nom=Application.langue.getProperty("nomsrl")+"°"+numero;
		statut=Application.langue.getProperty("statut");
		rd = new RechercheLocale(graphe.construireMatriceIntervalle(), mr, epsilon,numero);
	}
	
	public StartRechercheLocale(Graphe g,ArrayList<MethodeResolution> mr,double epsilon,JButton x)
	{
		super();
		graphe=g;
		num++;
		numero=num;
		nom=Application.langue.getProperty("nomsrl")+"°"+numero;
		statut=Application.langue.getProperty("statut");
		rd = new RechercheLocale(graphe.construireMatriceIntervalle(), mr, epsilon,numero);
		jb=x;
	}
	
	@SuppressWarnings("static-access")
	public void run()
	{
		rd.start();
		while(!rd.s.tryAcquire())
		{
			if(isInterrupted())
				break;
			jb.setEnabled(false);
		}
		if(!isInterrupted())
		{
			m=rd.resultat;
			jb.setEnabled(true);
			DessinGraphe dg = FenetrePrincipale.getInstance().getEspaceTravail().getDessinGraphe();
			if(dg.resFile)
				dg.createFileResult(m);
			statut=Application.langue.getProperty("statutfin");
			FenetrePrincipale.getInstance().getEspaceTravail().getMenuTraitement().majThread();
		}
		else
		{
			jb.setEnabled(true);
		}
	}
	
	public void AfficherResultat()
	{
		if(m!=null)
		{
			DessinGraphe dg = FenetrePrincipale.getInstance().getEspaceTravail().getDessinGraphe();
			if(!dg.resGraphe)
				new DialogAfficheMatrice(m,numero);
			else
			{
				graphe.setMatriceResultat(m);
				dg.repaint();
			}
		}
		else
			JOptionPane.showMessageDialog(FenetrePrincipale.getInstance(),Application.langue.getProperty("resultsrl1")+"°"+numero+" "+Application.langue.getProperty("resultsrl2"),Application.langue.getProperty("resultsrl3"),JOptionPane.WARNING_MESSAGE);
		
	}
	
	public void interrupt()
	{
		rd.interrupt();
	}
	

}
