/*
Copyright Bourdon Jeremie 
contributors : Giraudet Etienne, Docet Eric

jeremie.bourdon@univ-nantes.fr
eric.docet@etu.univ-nantes.fr
etienne.giraudet@etu.univ-nants.fr

This software is a computer program whose purpose is to do computations
on genetic graphs.

This software is governed by the CeCILL license under French law and
abiding by the rules of distribution of free software.  You can use, 
modify and/ or redistribute the software under the terms of the CeCILL
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL license and that you accept its terms.
*/

package thread;

import graphe.Graphe;
import ihm.FenetrePrincipale;
import ihm.intervalle.Intervalle;

import java.awt.Color;
import java.util.ArrayList;

import javax.swing.JButton;
import javax.swing.JSlider;

import util.JFloatField;

import calcul.Matrice;
import calcul.MethodeResolution;
import calcul.RechercheLocale;
import dialog.FenetreSimulation;
/**
 * Classe representant le calcul d'une solution dans la fenetre de simulation
 * @author giraudet
 *
 */
public class StartRechercheLocaleSimulation extends Thread{

	private RechercheLocale rl;
	private Matrice<JSlider> _ms;
	private Matrice<Boolean> _mb;
	private ArrayList<JFloatField> lf;
	private ArrayList<JFloatField> lfc;
	private ArrayList<JFloatField> lfmr;
	private Graphe _g;
	private JButton jb;
	private FenetreSimulation fs;
	private ArrayList<Boolean> methact;
	private static int num=0;
	private int numero;
	
	public StartRechercheLocaleSimulation(FenetreSimulation _fs,Matrice<Intervalle> mi,ArrayList<MethodeResolution> mr,double epsilon,Matrice<JSlider> ms, Matrice<Boolean> mb, ArrayList<JFloatField> _lf,ArrayList<JFloatField> _lfc,ArrayList<JFloatField> _lfmr,ArrayList<Boolean> methodeActive,Graphe g,JButton recherche)
	{
		num++;
		numero=num;
		rl=new RechercheLocale(mi,mr,epsilon,numero);
		_g=g;
		_ms=ms;
		_mb = mb;
		lf=_lf;
		lfc=_lfc;
		jb=recherche;
		fs=_fs;
		lfmr=_lfmr;
		methact = methodeActive;
	}
	
	
	public StartRechercheLocaleSimulation(FenetreSimulation _fs,Matrice<Intervalle> mi,ArrayList<MethodeResolution> mr,double epsilon,Matrice<JSlider> ms, Matrice<Boolean> mb, ArrayList<JFloatField> _lf,ArrayList<JFloatField> _lfc,ArrayList<JFloatField> _lfmr,ArrayList<Boolean> methodeActive,Graphe g)
	{
		num++;
		numero=num;
		rl=new RechercheLocale(mi,mr,epsilon,numero);
		_g=g;
		_ms=ms;
		_mb = mb;
		lf=_lf;
		lfc=_lfc;
		fs=_fs;
		lfmr=_lfmr;
		methact = methodeActive;
	}
	
	public void run()
	{
		rl.start();
		while(!rl.s.tryAcquire())
		{
		}
		Matrice<Double> mresult=rl.resultat;
		//on recuperer le resultat
		ArrayList<Double> lresult= rl.getVecteurKCalcule();
		if(mresult!=null)
		{
			//on bouge les sliders pour afficher la matrice solution trouvée
			for(int i=0;i<mresult.getNbLigne();i++)
				for(int j=0;j<mresult.getNbColonne();j++)
					_ms.getCase(i,j).setValue((int)Math.round((mresult.getCase(i, j)*1000)));
			//on affiche les résultats pour les coûts
			//on met vert quand la matrice est une solution
			//rouge quand ce n'est pas le cas
			for(int i=0;i<lf.size();i++)
			{
				if(_g.getlisteMatriceCoutP1().get(i).getActivation())
				{
					lf.get(i).setBackground(Color.GREEN);
					String s = lresult.get(i).toString();
					if(s.length()<=6)
						lf.get(i).setText(s);
					else
					{
						s=s.substring(0,6);
						lf.get(i).setText(s);
					}
				}
			}
			for(int i=0;i<lfc.size();i++)
			{
				if(FenetrePrincipale.getInstance().getEspaceTravail().getDessinGraphe().getChemins().get(i).getActivation())
				{
					lfc.get(i).setBackground(Color.GREEN);
					String s = lresult.get(i).toString();
					if(s.length()<=6)
						lfc.get(i).setText(s);
					else
					{
						s=s.substring(0,6);
						lfc.get(i).setText(s);
					}
				}
			}
			for(int i=0;i<lfmr.size();i++)
			{
				if(methact.get(i))
				{
					lfmr.get(i).setBackground(Color.GREEN);
					String s = lresult.get(i).toString();
					if(s.length()<=6)
						lfmr.get(i).setText(s);
					else
					{
						s=s.substring(0,6);
						lfmr.get(i).setText(s);
					}
				}
			}
		}
		else
		{
			for(int i=0;i<lf.size();i++)
				lf.get(i).setBackground(Color.RED);
			for(int i=0;i<lfc.size();i++)
				lfc.get(i).setBackground(Color.RED);
			for(int i=0;i<lfmr.size();i++)
				if(methact.get(i))
					lfmr.get(i).setBackground(Color.RED);
		}
		if(jb!=null)
			jb.setEnabled(true);
		for(int i=0;i<_mb.getNbLigne();i++)
			for(int j=0;j<_mb.getNbColonne();j++)
			{
				_mb.setCase(i, j, false);
				_ms.getCase(i, j).setEnabled(true);
			}
		fs.repaint();
	}
}
