/*
Copyright Bourdon Jeremie 
contributors : Giraudet Etienne, Docet Eric

jeremie.bourdon@univ-nantes.fr
eric.docet@etu.univ-nantes.fr
etienne.giraudet@etu.univ-nants.fr

This software is a computer program whose purpose is to do computations
on genetic graphs.

This software is governed by the CeCILL license under French law and
abiding by the rules of distribution of free software.  You can use, 
modify and/ or redistribute the software under the terms of the CeCILL
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL license and that you accept its terms.
*/

package util;

import calcul.Matrice;

import java.util.ArrayList;

/**
 * Classe servant a representer une file first-in first-out
 * @author giraudet-docet
 *
 * @param <E>
 */
public class ListeFIFO<E> 
{
	private ArrayList<E> liste;
	private int taille;
	
	public ListeFIFO(int n)
	{
		taille = n;
		liste = new ArrayList<E>();
	}
	
	public void add(E elt)
	{
		if(liste.size()==taille)
			liste.remove(0);
		liste.add(elt);
	}
	
	public int sizeMax()
	{
		return taille;
	}
	
	public boolean contains(E elt)
	{
		return liste.contains(elt);
	}
	
	public String toString()
	{
		return liste.toString();
	}
	
	/**
	 * la file contient que des matrices de double
	 * @param m
	 * @param epsilon
	 * @return
	 */
	@SuppressWarnings("unchecked")
	public boolean contientMatriceDouble(Matrice<Double> m,double epsilon)
	{
		for(int i =0;i<this.liste.size();i++)
		{
			if(Matrice.CompareMatriceDouble(m,(Matrice<Double>)this.liste.get(i),epsilon))
				return true;
		}
		return false;
	}
}
